# -*- coding: utf-8 -*-
"""test we can create an emailing from search results"""

from __future__ import unicode_literals

from unittest import skipIf

from django.conf import settings
from django.core.urlresolvers import reverse
from django.test.utils import override_settings

from coop_cms.models import Newsletter
from coop_cms.tests import BeautifulSoup
from model_mommy import mommy

from balafon.Crm import models

from balafon.Emailing.models import Emailing
from balafon.Search.tests import BaseTestCase


@override_settings(BALAFON_EMAILING_SENDER_CHOICES=None)
class CreateEmailingTest(BaseTestCase):
    """Test newsletter creation"""

    def test_view_new_emailing(self):
        """test view form when BALAFON_EMAILING_SENDER_CHOICES is not set"""
        contact1 = mommy.make(models.Contact, lastname="ABCD", main_contact=True, has_left=False)

        mommy.make(Newsletter)
        mommy.make(models.SubscriptionType)

        # TODO
        settings.BALAFON_EMAILING_SENDER_CHOICES = []

        data = {
            'contacts': [contact1.id]
        }

        url = reverse('search_emailing')
        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)

        node = soup.select("#id_from_email")[0]
        self.assertEqual("hidden", node["type"])

    def test_view_new_emailing_from_email(self):
        """test view form when BALAFON_EMAILING_SENDER_CHOICES is set"""
        contact1 = mommy.make(models.Contact, lastname="ABCD", main_contact=True, has_left=False)

        settings.BALAFON_EMAILING_SENDER_CHOICES = (
            ('toto@toto.fr', 'Toto',),
            ('titi@titi.com', 'Titi'),
        )

        mommy.make(Newsletter)
        mommy.make(models.SubscriptionType)

        data = {
            'contacts': [contact1.id]
        }

        url = reverse('search_emailing')
        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)

        nodes = soup.select("#id_from_email option")
        self.assertEqual(2, len(nodes))
        self.assertEqual(settings.BALAFON_EMAILING_SENDER_CHOICES[0][0], nodes[0]["value"])
        self.assertEqual(settings.BALAFON_EMAILING_SENDER_CHOICES[0][1], nodes[0].text)
        self.assertEqual(settings.BALAFON_EMAILING_SENDER_CHOICES[1][0], nodes[1]["value"])
        self.assertEqual(settings.BALAFON_EMAILING_SENDER_CHOICES[1][1], nodes[1].text)

    def test_create_emailing(self):
        """test create an emailing"""
        contact1 = mommy.make(models.Contact, lastname="ABCD", main_contact=True, has_left=False)
        contact2 = mommy.make(models.Contact, lastname="EFGH", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, lastname="IJKL", main_contact=True, has_left=False)

        newsletter = mommy.make(Newsletter)
        subscription_type = mommy.make(models.SubscriptionType)

        data = {
            'create_emailing': True,
        'subject': "",
            'subscription_type': subscription_type.id,
            'newsletter': newsletter.id,
            'contacts': ";".join([str(x) for x in [contact1.id, contact2.id]]),
            'lang': '',
            'from_email': '',
        }

        url = reverse('search_emailing')
        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)
        self.assertContains(
            response,
            '<script>$.colorbox.close(); window.location="{0}";</script>'.format(newsletter.get_absolute_url())
        )

        self.assertEqual(Emailing.objects.count(), 1)
        emailing = Emailing.objects.all()[0]

        self.assertEqual(emailing.subscription_type, subscription_type)
        self.assertEqual(emailing.newsletter, newsletter)
        self.assertEqual(emailing.from_email, "")
        self.assertEqual(0, emailing.sent_to.count())
        self.assertEqual(2, emailing.send_to.count())
        self.assertTrue(contact1 in emailing.send_to.all())
        self.assertTrue(contact2 in emailing.send_to.all())
        self.assertFalse(contact3 in emailing.send_to.all())

    def test_create_emailing_anonymous(self):
        """test create an emailing: anonymous user"""
        contact1 = mommy.make(models.Contact, lastname="ABCD", main_contact=True, has_left=False)
        contact2 = mommy.make(models.Contact, lastname="EFGH", main_contact=True, has_left=False)

        newsletter = mommy.make(Newsletter)
        subscription_type = mommy.make(models.SubscriptionType)

        data = {
            'create_emailing': True,
            'subject': "",
            'subscription_type': subscription_type.id,
            'newsletter': newsletter.id,
            'contacts': ";".join([str(x) for x in [contact1.id, contact2.id]]),
            'lang': '',
            'from_email': '',
        }

        self.client.logout()

        url = reverse('search_emailing')
        response = self.client.post(url, data=data)
        self.assertEqual(302, response.status_code)

        self.assertEqual(Emailing.objects.count(), 0)

    def test_create_emailing_not_in_staff(self):
        """test create an emailing: anonymous user"""
        contact1 = mommy.make(models.Contact, lastname="ABCD", main_contact=True, has_left=False)
        contact2 = mommy.make(models.Contact, lastname="EFGH", main_contact=True, has_left=False)
        mommy.make(models.Contact, lastname="IJKL", main_contact=True, has_left=False)

        newsletter = mommy.make(Newsletter)
        subscription_type = mommy.make(models.SubscriptionType)

        data = {
            'create_emailing': True,
            'subject': "",
            'subscription_type': subscription_type.id,
            'newsletter': newsletter.id,
            'contacts': ";".join([str(x) for x in [contact1.id, contact2.id]]),
            'lang': '',
            'from_email': '',
        }

        self.user.is_staff = False
        self.user.save()

        url = reverse('search_emailing')
        response = self.client.post(url, data=data)
        self.assertEqual(302, response.status_code)

        self.assertEqual(Emailing.objects.count(), 0)

    def test_create_emailing_from_email(self):
        """test create with from_email set"""
        contact1 = mommy.make(models.Contact, lastname="ABCD", main_contact=True, has_left=False)
        contact2 = mommy.make(models.Contact, lastname="EFGH", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, lastname="IJKL", main_contact=True, has_left=False)

        settings.BALAFON_EMAILING_SENDER_CHOICES = (
            ('Toto', 'toto@toto.fr'),
            ('Titi', 'titi@titi.com'),
        )

        newsletter = mommy.make(Newsletter)
        subscription_type = mommy.make(models.SubscriptionType)

        data = {
            'create_emailing': True,
            'subject': "",
            'subscription_type': subscription_type.id,
            'newsletter': newsletter.id,
            'contacts': ";".join([str(x) for x in [contact1.id, contact2.id]]),
            'lang': '',
            "from_email": "toto@toto.fr"
        }

        url = reverse('search_emailing')
        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)
        self.assertContains(
            response,
            '<script>$.colorbox.close(); window.location="{0}";</script>'.format(newsletter.get_absolute_url())
        )

        self.assertEqual(Emailing.objects.count(), 1)
        emailing = Emailing.objects.all()[0]

        self.assertEqual(emailing.subscription_type, subscription_type)
        self.assertEqual(emailing.newsletter, newsletter)
        self.assertEqual('', emailing.lang)
        self.assertEqual(emailing.from_email, "toto@toto.fr")
        self.assertEqual(0, emailing.sent_to.count())
        self.assertEqual(2, emailing.send_to.count())
        self.assertTrue(contact1 in emailing.send_to.all())
        self.assertTrue(contact2 in emailing.send_to.all())
        self.assertFalse(contact3 in emailing.send_to.all())

    def test_create_emailing_new_newsletter(self):
        """create emailing with new newsletter set"""
        contact1 = mommy.make(models.Contact, lastname="ABCD", main_contact=True, has_left=False)
        contact2 = mommy.make(models.Contact, lastname="EFGH", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, lastname="IJKL", main_contact=True, has_left=False)

        subscription_type = mommy.make(models.SubscriptionType)

        data = {
            'create_emailing': True,
            'subject': "Test",
            'subscription_type': subscription_type.id,
            'newsletter': 0,
            'contacts': ";".join([str(x) for x in [contact1.id, contact2.id]]),
            'lang': '',
            'from_email': '',
        }

        url = reverse('search_emailing')
        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertEqual(Newsletter.objects.count(), 1)
        newsletter = Newsletter.objects.all()[0]

        self.assertContains(
            response,
            '<script>$.colorbox.close(); window.location="{0}";</script>'.format(newsletter.get_absolute_url())
        )

        self.assertEqual(Emailing.objects.count(), 1)
        emailing = Emailing.objects.all()[0]

        self.assertEqual(emailing.subscription_type, subscription_type)
        self.assertEqual(emailing.newsletter, newsletter)
        self.assertEqual(emailing.newsletter.subject, data["subject"])
        self.assertEqual('', emailing.lang)
        self.assertEqual(0, emailing.sent_to.count())
        self.assertEqual(2, emailing.send_to.count())
        self.assertTrue(contact1 in emailing.send_to.all())
        self.assertTrue(contact2 in emailing.send_to.all())
        self.assertFalse(contact3 in emailing.send_to.all())

    def test_create_emailing_invalid_subscription(self):
        """create emailing invalid subscription type"""
        contact1 = mommy.make(models.Contact, lastname="ABCD", main_contact=True, has_left=False)
        contact2 = mommy.make(models.Contact, lastname="EFGH", main_contact=True, has_left=False)
        mommy.make(models.Contact, lastname="IJKL", main_contact=True, has_left=False)

        data = {
            'create_emailing': True,
            'subject': "Test",
            'subscription_type': 0,
            'newsletter': 0,
            'contacts': ";".join([str(x) for x in [contact1.id, contact2.id]]),
            'lang': '',
            'from_email': '',
        }

        url = reverse('search_emailing')
        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertEqual(Newsletter.objects.count(), 0)
        self.assertEqual(Emailing.objects.count(), 0)

    @skipIf(len(settings.LANGUAGES) < 2, "LANGUAGES less than 2")
    def test_create_emailing_language(self):
        """create emailing language is set"""
        subscription_type = mommy.make(models.SubscriptionType)

        contact1 = mommy.make(models.Contact, lastname="ABCD", main_contact=True, has_left=False)
        contact2 = mommy.make(models.Contact, lastname="EFGH", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, lastname="IJKL", main_contact=True, has_left=False)

        data = {
            'create_emailing': True,
            'subject': "Test",
            'subscription_type': subscription_type.id,
            'newsletter': 0,
            'contacts': ";".join([str(x) for x in [contact1.id, contact2.id]]),
            'lang': settings.LANGUAGES[1][0],
        }

        url = reverse('search_emailing')
        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertEqual(Newsletter.objects.count(), 1)
        newsletter = Newsletter.objects.all()[0]

        self.assertContains(
            response,
            '<script>$.colorbox.close(); window.location="{0}";</script>'.format(newsletter.get_absolute_url())
        )

        self.assertEqual(Emailing.objects.count(), 1)
        emailing = Emailing.objects.all()[0]

        self.assertEqual(emailing.subscription_type, subscription_type)
        self.assertEqual(emailing.newsletter, newsletter)
        self.assertEqual(emailing.newsletter.subject, data["subject"])
        self.assertEqual(settings.LANGUAGES[1][0], emailing.lang)
        self.assertEqual(0, emailing.sent_to.count())
        self.assertEqual(2, emailing.send_to.count())
        self.assertTrue(contact1 in emailing.send_to.all())
        self.assertTrue(contact2 in emailing.send_to.all())
        self.assertFalse(contact3 in emailing.send_to.all())

    @override_settings(LANGUAGES=[('fr', 'Francais'), ('en', 'English')])
    def test_view_new_emailing_language(self):
        """view emailing with several languages: language is a select"""
        mommy.make(models.SubscriptionType)

        contact1 = mommy.make(models.Contact, lastname="ABCD", main_contact=True, has_left=False)
        contact2 = mommy.make(models.Contact, lastname="EFGH", main_contact=True, has_left=False)

        data = {
            'contacts': ";".join([str(x) for x in [contact1.id, contact2.id]]),
        }

        url = reverse('search_emailing')
        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        self.assertEqual(1, len(soup.select("select#id_lang")))

    @override_settings(LANGUAGES=[('fr', 'Francais')])
    def test_view_new_emailing_language_hidden(self):
        """view emailing with just one language: The language field is hidden"""
        mommy.make(models.SubscriptionType)

        contact1 = mommy.make(models.Contact, lastname="ABCD", main_contact=True, has_left=False)
        contact2 = mommy.make(models.Contact, lastname="EFGH", main_contact=True, has_left=False)

        data = {
            'contacts': ";".join([str(x) for x in [contact1.id, contact2.id]]),
        }

        url = reverse('search_emailing')
        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        self.assertEqual(0, len(soup.select("select#id_lang")))
        self.assertEqual(1, len(soup.select("#id_lang")))
        self.assertEqual("hidden", soup.select("#id_lang")[0]["type"])



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-04-04 15:25
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0027_auto_20190328_1336'),
    ]

    operations = [
        migrations.CreateModel(
            name='CustomFieldChoice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.CharField(max_length=100, verbose_name='value')),
                ('label', models.CharField(max_length=100, verbose_name='label')),
                ('order', models.IntegerField(default=0, verbose_name='order')),
            ],
            options={
                'verbose_name': 'custom field choice',
                'verbose_name_plural': 'custom field choices',
                'ordering': ('order', 'label'),
            },
        ),
        migrations.AddField(
            model_name='customfield',
            name='choices',
            field=models.ManyToManyField(blank=True, to='Crm.CustomFieldChoice'),
        ),
    ]



# -*- coding: utf-8 -*-
"""opportunities : certainly a bad name :-) this is a group of actions"""

from __future__ import unicode_literals

from datetime import datetime
import json

from django.core.urlresolvers import reverse
from django.contrib.auth.decorators import user_passes_test
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render, get_object_or_404
from django.utils.translation import ugettext as _

from colorbox.decorators import popup_redirect
from coop_cms.utils import paginate

from balafon.Crm import models, forms
from balafon.Crm.utils import get_actions_by_set
from balafon.permissions import can_access
from balafon.utils import log_error


@user_passes_test(can_access)
@log_error
def get_opportunity_name(request, opp_id):
    """view"""
    try:
        opp = models.Opportunity.objects.get(id=opp_id)
        return HttpResponse(json.dumps({'name': opp.name}), 'application/json')
    except (models.Opportunity.DoesNotExist, ValueError):
        return HttpResponse(json.dumps({'name': opp_id}), 'application/json')


@user_passes_test(can_access)
@log_error
def get_opportunities(request):
    """view"""
    term = request.GET.get('term')
    queryset = models.Opportunity.objects.filter(ended=False, name__icontains=term)
    opportunities = [{'id': x.id, 'name': '{0}'.format(x.name)} for x in queryset]
    return HttpResponse(json.dumps(opportunities), 'application/json')


@user_passes_test(can_access)
@log_error
def get_opportunity_id(request):
    """view"""
    name = request.GET.get('name')
    opportunity = get_object_or_404(models.Opportunity, name=name)
    return HttpResponse(json.dumps({'id': opportunity.id}), 'application/json')


@user_passes_test(can_access)
def view_entity_opportunities(request, entity_id):
    """view"""
    entity = get_object_or_404(models.Entity, id=entity_id)
    opportunities = models.Opportunity.objects.filter(entity=entity)

    request.session["redirect_url"] = reverse('crm_entity_opportunities', args=[entity_id])

    return render(
        request,
        'Crm/entity_opportunities.html',
        {
            'entity': entity,
            'opportunities': opportunities,
            'all_opportunities': True,
        },
    )


@user_passes_test(can_access)
def view_all_opportunities(request, ordering=None):
    """view"""
    opportunities = models.Opportunity.objects.all()
    if not ordering:
        ordering = 'date'
    if ordering == 'name':
        opportunities = opportunities.order_by('name')
    elif ordering == 'status':
        opportunities = opportunities.order_by('status__ordering', 'status')
    elif ordering == 'type':
        opportunities = opportunities.order_by('type')
    elif ordering == 'date':
        opportunities = list(opportunities)
        opportunities.sort(key=lambda o: o.get_start_date() or datetime(1970, 1, 1))
        opportunities.reverse()

    request.session["redirect_url"] = reverse('crm_all_opportunities')
    page_obj = paginate(request, opportunities, 50)

    return render(
        request,
        'Crm/all_opportunities.html',
        {
            "opportunities": list(page_obj),
            'page_obj': page_obj,
            "ordering": ordering,
            "all_opportunities": True,
        },
    )


@user_passes_test(can_access)
@popup_redirect
def edit_opportunity(request, opportunity_id):
    """view"""
    opportunity = get_object_or_404(models.Opportunity, id=opportunity_id)

    if request.method == 'POST':
        form = forms.OpportunityForm(request.POST, instance=opportunity)
        if form.is_valid():
            opportunity = form.save()
            next_url = request.session.get('redirect_url') or reverse('crm_view_opportunity', args=[opportunity.id])
            return HttpResponseRedirect(next_url)
    else:
        form = forms.OpportunityForm(instance=opportunity)

    return render(
        request,
        'Crm/edit_opportunity.html',
        {'opportunity': opportunity, 'form': form},
    )


@user_passes_test(can_access)
def view_opportunity(request, opportunity_id):
    """view"""
    opportunity = get_object_or_404(models.Opportunity, id=opportunity_id)
    actions = opportunity.action_set.filter(archived=False)
    actions_by_set = get_actions_by_set(actions)

    contacts = []
    for action in actions:
        contacts += [contact for contact in action.contacts.all()]
        for entity in action.entities.all():
            contacts += [contact for contact in entity.contact_set.filter(has_left=False)]
    contacts = list(set(contacts))
    contacts.sort(key=lambda contact_elt: contact_elt.lastname.lower())

    request.session["redirect_url"] = reverse('crm_view_opportunity', args=[opportunity.id])

    context = {
        'opportunity': opportunity,
        'actions_by_set': actions_by_set,
        'contacts': contacts,
    }

    return render(
        request,
        'Crm/view_opportunity.html',
        context,
    )


@user_passes_test(can_access)
@popup_redirect
def delete_opportunity(request, opportunity_id):
    """view"""
    opportunity = get_object_or_404(models.Opportunity, id=opportunity_id)

    if request.method == 'POST':
        form = forms.ConfirmForm(request.POST)
        if form.is_valid():
            if form.cleaned_data["confirm"]:
                for action in opportunity.action_set.all():
                    action.opportunity = None
                    action.save()
                opportunity.delete()
                next_url = reverse('crm_board_panel')
                return HttpResponseRedirect(next_url)
        else:
            return HttpResponseRedirect(reverse('crm_view_opportunity', args=[opportunity.id]))
    else:
        form = forms.ConfirmForm()

    return render(
        request,
        'balafon/confirmation_dialog.html',
        {
            'form': form,
            'message': _('Are you sure to delete the opportunity "{0}"?').format(opportunity),
            'action_url': reverse("crm_delete_opportunity", args=[opportunity_id]),
        },
    )


@user_passes_test(can_access)
@popup_redirect
def add_action_to_opportunity(request, action_id):
    """view"""
    action = get_object_or_404(models.Action, id=action_id)

    if request.method == "POST":
        form = forms.SelectOpportunityForm(request.POST)
        if form.is_valid():
            opportunity = form.cleaned_data["opportunity"]
            action.opportunity = opportunity
            action.save()
            next_url = request.session.get('redirect_url')
            next_url = next_url or reverse('crm_view_opportunity', args=[opportunity.id])
            return HttpResponseRedirect(next_url)
    else:
        form = forms.SelectOpportunityForm()

    return render(
        request,
        'Crm/add_action_to_opportunity.html',
        {'action': action, 'form': form},
    )


@user_passes_test(can_access)
@popup_redirect
def remove_action_from_opportunity(request, action_id, opportunity_id):
    """view"""
    action = get_object_or_404(models.Action, id=action_id)
    opportunity = get_object_or_404(models.Opportunity, id=opportunity_id)

    if request.method == "POST":
        form = forms.ConfirmForm(request.POST)
        if form.is_valid():
            if form.cleaned_data["confirm"]:
                if action.opportunity == opportunity:
                    action.opportunity = None
                    action.save()
            next_url = request.session.get('redirect_url')
            next_url = next_url or reverse('crm_view_opportunity', args=[opportunity.id])
            return HttpResponseRedirect(next_url)
    else:
        form = forms.ConfirmForm()

    return render(
        request,
        'balafon/confirmation_dialog.html',
        {
            'form': form,
            'message': _('Do you want to remove the action {0} from opportunity {1}?').format(
                action.subject, opportunity.name
            ),
            'action_url': reverse("crm_remove_action_from_opportunity", args=[action.id, opportunity.id]),
        },
    )


@user_passes_test(can_access)
@popup_redirect
def add_opportunity(request):
    """view"""
    next_url = request.session.get('redirect_url')
    if request.method == 'POST':
        opportunity = models.Opportunity()
        form = forms.OpportunityForm(request.POST, instance=opportunity)
        if form.is_valid():
            opportunity = form.save()
            next_url = next_url or reverse('crm_view_opportunity', args=[opportunity.id])
            return HttpResponseRedirect(next_url)
    else:
        form = forms.OpportunityForm()

    next_url = next_url or reverse('crm_board_panel')
    return render(
        request,
        'Crm/edit_opportunity.html',
        {'next_url': next_url, 'form': form},
    )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-05-09 22:52
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('Crm', '0028_auto_20190404_1525'),
    ]

    operations = [
        migrations.AddField(
            model_name='subscriptiontype',
            name='allowed_on_sites',
            field=models.ManyToManyField(blank=True, related_name='_subscriptiontype_allowed_on_sites_+', to='sites.Site'),
        ),
    ]



# -*- coding: utf-8 -*-
"""view and edit entities"""

from __future__ import unicode_literals, print_function

from datetime import date
import json
import re

from django.contrib.auth.decorators import user_passes_test
from django.contrib.messages import warning
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.http import HttpResponse, Http404, HttpResponseRedirect
from django.shortcuts import render, get_object_or_404
from django.utils.translation import ugettext as _

from coop_cms.utils import paginate
from colorbox.decorators import popup_redirect

from balafon.Crm import models, forms
from balafon.Crm.utils import get_actions_by_set
from balafon.Crm.settings import is_unaccent_filter_supported
from balafon.permissions import can_access
from balafon.utils import log_error


@user_passes_test(can_access)
def get_entity_name(request, entity_id):
    """view"""
    try:
        entity = models.Entity.objects.get(id=entity_id)
        return HttpResponse(json.dumps({'name': entity.name}), 'application/json')
    except models.Entity.DoesNotExist:
        return HttpResponse(json.dumps({'name': entity_id}), 'application/json')


@user_passes_test(can_access)
def get_entities(request):
    """view"""
    term = request.GET.get('term')
    entities = [
        {'id': x.id, 'name': x.name}
        for x in models.Entity.objects.filter(name__icontains=term)
    ]
    return HttpResponse(json.dumps(entities), 'application/json')


@user_passes_test(can_access)
@log_error
def get_entity_id(request):
    """view"""
    name = request.GET.get('name', '')
    if name:
        try:
            entity = get_object_or_404(models.Entity, name=name)
        except models.Entity.MultipleObjectsReturned:
            warning(request, _('Duplicate entities {0}').format(name))
            raise Http404
        return HttpResponse(json.dumps({'id': entity.id}), 'application/json')
    raise Http404


@user_passes_test(can_access)
def view_entity(request, entity_id):
    """view"""

    try:
        preview = bool(request.GET.get('preview', False))
    except ValueError:
        preview = False

    entity = get_object_or_404(models.Entity, id=entity_id)
    contacts = entity.contact_set.all().order_by("has_left", "-main_contact", "lastname", "firstname")

    actions = models.Action.objects.filter(
        Q(entities=entity) | Q(contacts__entity=entity), Q(archived=False)
    ).distinct().order_by("planned_date", "priority")

    actions_by_set = get_actions_by_set(actions, 5)

    multi_user = True
    request.session["redirect_url"] = reverse('crm_view_entity', args=[entity_id])

    context = {
        "entity": entity,
        'contacts': contacts,
        'actions_by_set': actions_by_set,
        'multi_user': multi_user,
        'preview': preview,
        'template_base': "balafon/bs_base.html" if not preview else "balafon/bs_base_raw.html"
    }

    return render(
        request,
        'Crm/entity.html',
        context,
    )


@user_passes_test(can_access)
def view_entities_list(request):
    """view"""
    letter_filter = request.GET.get("filter", "*")

    queryset = models.Entity.objects.all()
    if re.search(r"\w+", letter_filter):
        if is_unaccent_filter_supported():
            queryset = queryset.extra(
                where=["UPPER(unaccent(name)) LIKE UPPER(unaccent(%s))"],
                params=["{0}%".format(letter_filter)]
            )
        else:
            queryset = queryset.filter(name__istartswith=letter_filter)
    elif letter_filter == "~":
        queryset = queryset.filter(name__regex=r'^\W|^\d')

    queryset = queryset.extra(select={'lower_name':'lower(name)'}).order_by('lower_name')

    page_obj = paginate(request, queryset, 50)

    entities = list(page_obj)

    return render(
        request,
        'Crm/all_entities.html',
        {
            'entities': entities,
            'letter_filter': letter_filter,
            'page_obj': page_obj,
        },
    )


@user_passes_test(can_access)
@popup_redirect
def change_contact_entity(request, contact_id):
    """view"""
    try:
        contact = get_object_or_404(models.Contact, id=contact_id)

        if request.method == "POST":
            form = forms.ChangeContactEntityForm(contact, request.POST)
            if form.is_valid():
                form.change_entity()
                next_url = reverse('crm_view_contact', args=[contact_id])
                return HttpResponseRedirect(next_url)
        else:
            form = forms.ChangeContactEntityForm(contact)

        context_dict = {
            'contact': contact,
            'form': form,
        }

        return render(
            request,
            'Crm/change_contact_entity.html',
            context_dict
        )
    except Exception as msg:
        print("#ERR", msg)
        raise



@user_passes_test(can_access)
@popup_redirect
def edit_entity(request, entity_id):
    """view"""
    entity = get_object_or_404(models.Entity, id=entity_id)
    #update last access
    entity.save()
    if request.method == "POST":
        form = forms.EntityForm(request.POST, request.FILES, instance=entity)
        if form.is_valid():
            entity = form.save()
            return HttpResponseRedirect(reverse('crm_view_entity', args=[entity.id]))
    else:
        form = forms.EntityForm(instance=entity)

    return render(
        request,
        'Crm/edit_entity.html',
        locals()
    )


@user_passes_test(can_access)
@popup_redirect
def create_entity(request, entity_type_id):
    """view"""
    try:
        entity_type_id = int(entity_type_id)
    except ValueError:
        raise Http404
    if entity_type_id:
        entity_type = get_object_or_404(models.EntityType, id=entity_type_id)
        entity = models.Entity(type=entity_type)
    else:
        entity = models.Entity()

    if request.method == "POST":
        form = forms.EntityForm(request.POST, request.FILES, instance=entity)
        if form.is_valid():
            entity = form.save()
            return HttpResponseRedirect(reverse('crm_view_entity', args=[entity.id]))
    else:
        form = forms.EntityForm(instance=entity, initial={'relationship_date': date.today()})

    return render(
        request,
        'Crm/edit_entity.html',
        {
            'entity': entity,
            'form': form,
            'create_entity': True,
            'entity_type_id': entity_type_id,
        }
    )


@user_passes_test(can_access)
@popup_redirect
def delete_entity(request, entity_id):
    """view"""
    entity = get_object_or_404(models.Entity, id=entity_id)
    if request.method == 'POST':
        form = forms.ConfirmForm(request.POST)
        if form.is_valid():
            if form.cleaned_data["confirm"]:
                entity.delete()
                return HttpResponseRedirect(reverse('balafon_homepage'))
            else:
                return HttpResponseRedirect(reverse('crm_edit_entity', args=[entity.id]))
    else:
        form = forms.ConfirmForm()
    return render(
        request,
        'balafon/confirmation_dialog.html',
        {
            'form': form,
            'message': _('Are you sure to delete {0.name}?').format(entity),
            'action_url': reverse("crm_delete_entity", args=[entity_id]),
        }
    )


@user_passes_test(can_access)
def select_entity_and_redirect(request, view_name, template_name):
    """view"""
    if request.method == 'POST':
        form = forms.SelectEntityForm(request.POST)
        if form.is_valid():
            entity = form.cleaned_data["entity"]
            args = [entity.id]
            url = reverse(view_name, args=args)
            return HttpResponseRedirect(url)
    else:
        form = forms.SelectEntityForm()

    return render(
        request,
        template_name,
        {'form': form}
    )



# -*- coding: utf-8 -*-
"""documents"""

from __future__ import unicode_literals

from django.core.exceptions import PermissionDenied
from django.conf import settings
from django.contrib.messages import api as user_message
from django.http import Http404
from django.shortcuts import get_object_or_404
from django.template import TemplateDoesNotExist
from django.template.defaultfilters import slugify
from django.template.loader import get_template
from django.utils.translation import ugettext as _

from coop_cms.generic_views import EditableObjectView
from wkhtmltopdf.views import PDFTemplateView

from balafon.Crm import models, forms
from balafon.utils import logger


class ActionDocumentDetailView(EditableObjectView):
    """view"""
    model = models.ActionDocument
    edit_mode = False
    form_class = forms.ActionDocumentForm
    #varname = "action_doc"

    def get_object(self):
        """get action"""
        action = get_object_or_404(models.Action, pk=self.kwargs['pk'])
        try:
            return action.actiondocument
        except self.model.DoesNotExist:
            warning_text = ""
            if not action.type:
                warning_text = _("The action has no type set: Unable to create the corresponding document")
            elif not action.type.default_template:
                warning_text = _(
                    "The action type has no document template defined: Unable to create the corresponding document"
                )
            if warning_text:
                logger.warning(warning_text)
                user_message.warning(self.request, warning_text)
                raise Http404
            else:
                return self.model.objects.create(action=action, template=action.type.default_template)

    def get_template(self):
        """get template"""
        return self.object.template


class ActionDocumentEditView(ActionDocumentDetailView):
    """view"""
    edit_mode = True


class ActionDocumentPdfView(PDFTemplateView):
    """view"""

    def find_template(self, template_type, action_type):
        """look for template"""
        potential_templates = []
        if action_type:
            action_type = slugify(action_type.name)
            potential_templates += [
                "documents/_{0}_{1}.html".format(action_type, template_type),
            ]

        potential_templates += [
            "documents/_{0}.html".format(template_type),
        ]

        for template_name in potential_templates:
            try:
                get_template(template_name)
                return template_name
            except TemplateDoesNotExist:
                pass

        return None

    def render_to_response(self, context, **response_kwargs):
        """render"""
        try:
            action = get_object_or_404(models.Action, pk=self.kwargs['pk'])
        except models.Action.DoesNotExist:
            raise Http404
        try:
            doc = action.actiondocument
        except models.ActionDocument.DoesNotExist:
            raise Http404

        if not self.request.user.has_perm('can_view_object', doc):
            raise PermissionDenied
        context['to_pdf'] = True
        context['object'] = doc
        self.template_name = doc.template

        pdf_options_dict = getattr(settings, 'BALAFON_PDF_OPTIONS', None)
        if pdf_options_dict is None:
            pdf_options = {'margin-top': 0, 'margin-bottom': 0, 'margin-right': 0, 'margin-left': 0, }
        else:
            pdf_options = pdf_options_dict.get(self.template_name, {})
        if self.cmd_options:
            self.cmd_options.update(pdf_options)
        else:
            self.cmd_options = pdf_options
        self.header_template = self.find_template("header", action.type)
        self.footer_template = self.find_template("footer", action.type)
        self.filename = slugify("{0}.contact - {0}.subject".format(action))+".pdf"
        return super(ActionDocumentPdfView, self).render_to_response(context, **response_kwargs)



# -*- coding: utf-8 -*-
"""common place for settings. Define defaults"""

from __future__ import unicode_literals

from django.conf import settings
from django.utils.translation import ugettext as _

from ..utils import load_from_module


def is_public_api_allowed():
    """returns true if an anonymous can view some data : products, categories, tags"""
    return getattr(settings, 'BALAFON_STORE_ALLOW_PUBLIC_API', False)


def get_cart_type_name():
    """returns true if an anonymous can view some data : products, categories, tags"""
    return getattr(settings, 'BALAFON_STORE_CART_TYPE', _('Cart'))


def get_cart_confirmation_subject(profile, action):
    """returns true if an anonymous can view some data : products, categories, tags"""
    subject = getattr(settings, 'BALAFON_CART_CONFIRMATION_EMAIL_SUBJECT', _('Purchasing confirmation'))
    if callable(subject):
        return subject(profile, action)
    else:
        return subject


def get_cart_processed_callback():
    """
    returns a callback defined in BALAFON_CART_PROCESSED_CALLBACK
    This function should take an action has argument
    """
    return load_from_module('BALAFON_CART_PROCESSED_CALLBACK', None)


def get_thumbnail_size():
    """
    returns size of image when displayed as thumbnail
    """
    return getattr(settings, 'BALAFON_STORE_THUMBNAIL_SIZE', "64x64")


def get_thumbnail_crop():
    """
    returns size of image when displayed as thumbnail
    """
    return getattr(settings, 'BALAFON_STORE_THUMBNAIL_CROP', "center")


def get_image_size():
    """
    returns size of image when displayed as thumbnail
    """
    return getattr(settings, 'BALAFON_STORE_IMAGE_SIZE', "300")


def get_image_crop():
    """
    returns size of image when displayed as thumbnail
    """
    return getattr(settings, 'BALAFON_STORE_IMAGE_CROP', None)



# -*- coding: utf-8 -*-
"""test we can search contact by subscription type"""

from __future__ import unicode_literals

from django.core.urlresolvers import reverse

from model_mommy import mommy

from balafon.Crm import models
from balafon.Search.tests import BaseTestCase


class SubscriptionSearchTest(BaseTestCase):
    """subscriptions"""

    def test_search_accept_subscription(self):
        """search by accept subscriptions"""
        mommy.make(models.SubscriptionType)

        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        url = reverse('search')

        subscription_type = mommy.make(models.SubscriptionType)

        mommy.make(
            models.Subscription, contact=contact1, subscription_type=subscription_type, accept_subscription=True
        )
        mommy.make(
            models.Subscription, contact=contact3, subscription_type=subscription_type, accept_subscription=False
        )

        data = {"gr0-_-accept_subscription-_-0": subscription_type.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertNotContains(response, contact3.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

    def test_search_refuse_subscription(self):
        """search by refuse subscription"""
        mommy.make(models.SubscriptionType)

        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        url = reverse('search')

        subscription_type = mommy.make(models.SubscriptionType)

        mommy.make(
            models.Subscription, contact=contact1, subscription_type=subscription_type, accept_subscription=True
        )
        mommy.make(
            models.Subscription, contact=contact3, subscription_type=subscription_type, accept_subscription=False
        )

        data = {"gr0-_-refuse_subscription-_-0": subscription_type.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertNotContains(response, contact1.lastname)
        self.assertContains(response, contact3.lastname)

        self.assertContains(response, entity2.name)
        self.assertContains(response, contact2.lastname)



# -*- coding: utf-8 -*-
"""Crm forms"""

from __future__ import unicode_literals

from datetime import datetime
from decimal import Decimal

from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils import timezone
from django.utils.translation import ugettext, ugettext_lazy as _

import floppyforms.__future__ as forms

from coop_cms.forms.base import InlineHtmlEditableModelForm

from balafon.widgets import CalcHiddenInput
from balafon.Crm import models
from balafon.Crm.forms.base import BetterBsModelForm, FormWithFieldsetMixin, BsPopupModelForm
from balafon.Crm.widgets import OpportunityAutoComplete


class ActionForm(FormWithFieldsetMixin, BetterBsModelForm):
    """form for creating or editing action"""

    date = forms.DateField(label=_(u"planned date"), required=False, widget=forms.TextInput())
    time = forms.TimeField(label=_(u"planned time"), required=False)

    end_date = forms.DateField(label=_(u"end date"), required=False, widget=forms.TextInput())
    end_time = forms.TimeField(label=_(u"end time"), required=False)

    amount = forms.DecimalField(label=_("Amount"), required=False)

    class Meta:
        """form from model"""
        model = models.Action
        fields = (
            'type', 'subject', 'date', 'time', 'status', 'status2', 'in_charge', 'detail',
            'amount', 'number', 'planned_date', 'end_date', 'end_time', 'end_datetime', 'opportunity'
        )
        fieldsets = [
            ('summary', {
                'fields': [
                    'subject', 'in_charge', 'date', 'time', 'planned_date', 'end_date', 'end_time', 'end_datetime',
                    'opportunity'
                ],
                'legend': _('Summary')
            }),
            ('type', {'fields': ['type', 'status', 'status2', 'amount', 'number'], 'legend': _('Type')}),
            ('details', {'fields': ['detail'], 'legend': _('Details')}),
        ]
        help_texts = {
            'amount': _('Amount is disabled when value is calculated'),
        }

    def __init__(self, *args, **kwargs):
        kwargs.pop('entity', None)
        instance = kwargs.get('instance', None)
        action_type = kwargs.pop('action_type', None)
        super(ActionForm, self).__init__(*args, **kwargs)
        self.title = u""
        if instance:
            action_type = instance.type
        self.action_type = action_type

        is_amount_calculated = False
        if action_type:
            is_amount_calculated = action_type.is_amount_calculated
            self.calculated_amount = Decimal("0")
            for fieldset_name, fieldset_attrs in self.Meta.fieldsets:
                if fieldset_name == 'type':
                    fieldset_attrs['legend'] = action_type.name
                    break
            self.fields['type'].widget = forms.HiddenInput()
            self.fields['type'].initial = action_type
            if instance:
                self.title = ugettext(u"Edition {0}").format(action_type.name)
            else:
                self.title = ugettext(u"Creation {0}").format(action_type.name)
        else:
            self.title = ugettext(u"Edit action") if instance else ugettext(u"Create action")

        is_auto_generated = (action_type and action_type.number_auto_generated) or \
                            (instance and instance.type and instance.type.number_auto_generated)
        if is_auto_generated:
            self.fields['number'].widget.attrs['disabled'] = 'disabled'
            self.fields['number'].required = False

        self.fields['amount'].widget.attrs['step'] = 'any'
        if is_amount_calculated:
            self.fields['amount'].widget.attrs['disabled'] = 'disabled'
        self.is_amount_calculated = is_amount_calculated

        if action_type and action_type.allowed_status.count():
            # let javascript disable the blank value if default_status
            choices = [('', "---------")]
            allowed_status = action_type.allowed_status.all()
            if instance and instance.frozen:
                allowed_status = allowed_status.filter(allowed_on_frozen=True)
            self.fields['status'].choices = choices + [
                (status.id, status.name) for status in allowed_status
            ]
            if action_type.default_status:
                self.fields['status'].initial = action_type.default_status.id
            else:
                self.fields['status'].initial = ''

        if action_type and action_type.allowed_status2.count():
            # let javascript disable the blank value if default_status2
            allowed_status2 = action_type.allowed_status2.all()
            if instance and instance.frozen:
                allowed_status2 = allowed_status2.filter(allowed_on_frozen=True)
            choices = [('', "---------")]
            self.fields['status2'].choices = choices + [
                (status.id, status.name) for status in allowed_status2
            ]
            if action_type.default_status2:
                self.fields['status2'].initial = action_type.default_status2.id
            else:
                self.fields['status2'].initial = ''
        else:
            self.fields['status2'].widget = forms.HiddenInput()

        self.fields['opportunity'].widget = forms.HiddenInput()
        self.fields['detail'].widget = forms.Textarea(attrs={'placeholder': _('enter details'), 'cols': '72'})

        self._init_dt_field("planned_date", "date", "time")
        self._init_dt_field("end_datetime", "end_date", "end_time")

    def _init_dt_field(self, dt_field, date_field, time_field):
        """init datetime fields"""
        self.fields[dt_field].widget = CalcHiddenInput()
        the_datetime = getattr(self.instance, dt_field) if self.instance else self.fields[dt_field].initial
        if the_datetime:
            self.fields[date_field].initial = the_datetime.date()
            if settings.USE_TZ:
                utc_dt = the_datetime.replace(tzinfo=timezone.utc)
                loc_dt = utc_dt.astimezone(timezone.get_current_timezone())
                self.fields[time_field].initial = loc_dt.time()
            else:
                self.fields[time_field].initial = the_datetime.time()

        is_frozen = self.instance.frozen if self.instance else False
        if is_frozen:
            self.fields[date_field].widget.attrs['disabled'] = 'disabled'
            self.fields[time_field].widget.attrs['disabled'] = 'disabled'

    def clean_status(self):
        """status validation"""
        type_of = self.cleaned_data.get('type') or self.action_type
        status = self.cleaned_data['status']
        if type_of:
            allowed_status = ([] if type_of.default_status else [None]) + list(type_of.allowed_status.all())
            if len(allowed_status) > 0 and status not in allowed_status:
                raise ValidationError(ugettext(u"This status can't not be used for this action type"))
        else:
            if status:
                raise ValidationError(ugettext(u"Please select a type before defining the status"))
        return status

    def clean_status2(self):
        """status validation"""
        type_of = self.cleaned_data['type']
        status = self.cleaned_data['status2']
        if type_of:
            allowed_status = ([] if type_of.default_status2 else [None]) + list(type_of.allowed_status2.all())
            if len(allowed_status) > 0 and status not in allowed_status:
                raise ValidationError(ugettext(u"This status can't not be used for this action type"))
        else:
            if status:
                raise ValidationError(ugettext(u"Please select a type before defining the status"))
        return status

    def clean_planned_date(self):
        """planned date validation"""
        the_date = self.cleaned_data.get("date", None)
        the_time = self.cleaned_data.get("time", None)
        if the_date:
            return datetime.combine(the_date, the_time or datetime.min.time())
        return None

    def clean_time(self):
        """time validation"""
        the_date = self.cleaned_data.get("date", None)
        the_time = self.cleaned_data.get("time", None)
        if the_time and not the_date:
            raise ValidationError(_(u"You must set a date"))
        return the_time

    def clean_end_date(self):
        """end date validation"""
        date1 = self.cleaned_data.get("date", None)
        date2 = self.cleaned_data.get("end_date", None)
        if date2:
            start_dt = self.cleaned_data["planned_date"]
            if not start_dt:
                raise ValidationError(_(u"The planned date is not defined"))
            if date1 > date2:
                raise ValidationError(_(u"The end date must be after the planned date"))
        return date2

    def clean_end_time(self):
        """end time validation"""
        date1 = self.cleaned_data.get("date", None)
        date2 = self.cleaned_data.get("end_date", None)
        time1 = self.cleaned_data.get("time", None)
        time2 = self.cleaned_data.get("end_time", None)

        if time2:
            if time2 and not date2:
                raise ValidationError(_(u"You must set a end date"))

            if date1 == date2 and (time1 or datetime.min.time()) >= time2:
                raise ValidationError(_(u"The end time must be after the planned time"))

        elif time1:
            if date1 == date2 and time1 >= datetime.min.time():
                raise ValidationError(_(u"The end time must be set"))
        return time2

    def clean_end_datetime(self):
        """clean end datetime"""
        end_date = self.cleaned_data.get("end_date", None)
        end_time = self.cleaned_data.get("end_time", None)
        if end_date:
            return datetime.combine(end_date, end_time or datetime.min.time())
        return None

    def clean_amount(self):
        if self.is_amount_calculated:
            return self.calculated_amount
        else:
            return self.cleaned_data['amount']

    def save(self, *args, **kwargs):
        return super(ActionForm, self).save(*args, **kwargs)


class ActionTypeForm(forms.ModelForm):
    """action type form"""

    class Meta:
        """form from model"""
        model = models.ActionType
        fields = (
            'subscribe_form', 'set', 'last_number', 'number_auto_generated', 'default_template', 'allowed_status',
            'default_status', 'is_editable', 'action_template', 'order_index', 'is_amount_calculated',
            'next_action_types', 'not_assigned_when_cloned', 'generate_uuid', 'hide_contacts_buttons',
            'mail_to_subject', 'allowed_status2', 'default_status2',
        )


class ActionDocumentForm(InlineHtmlEditableModelForm):
    """Action document form"""
    class Meta:
        model = models.ActionDocument
        fields = ('content',)


class ActionDoneForm(forms.ModelForm):
    """form setting an action done"""

    class Meta:
        """form from model"""
        model = models.Action
        fields = ['done']
        widgets = {
            'done': forms.HiddenInput(),
        }

    def __init__(self, *args, **kwargs):
        inst = kwargs.get('instance')
        inst.done = not inst.done
        kwargs['instance'] = inst
        super(ActionDoneForm, self).__init__(*args, **kwargs)


class OpportunityForm(FormWithFieldsetMixin, BetterBsModelForm):
    """opportunity form"""
    class Meta:
        """form from model"""
        model = models.Opportunity
        fields = ('name', 'detail')

        fieldsets = [
            ('name', {'fields': ['name', 'detail'], 'legend': _('Summary')}),
        ]

    def __init__(self, *args, **kwargs):
        super(OpportunityForm, self).__init__(*args, **kwargs)
        self.fields['detail'].widget = forms.Textarea(attrs={'placeholder': _('enter details'), 'cols':'72'})


class OpportunityStatusForm(forms.ModelForm):
    """opportunity status form"""

    class Meta:
        """form from model"""
        model = models.OpportunityStatus
        fields = ('name', 'ordering', )


class SelectOpportunityForm(forms.Form):
    """Select opportunity"""

    def __init__(self, *args, **kwargs):
        kwargs.pop('choices', None)
        super(SelectOpportunityForm, self).__init__(*args, **kwargs)
        widget = OpportunityAutoComplete(
            attrs={'placeholder': _('Enter the name of a opportunity'), 'size': '50', 'class': 'colorbox'})
        self.fields["opportunity"] = forms.CharField(label=_(u"Opportunity"), widget=widget)

    def clean_opportunity(self):
        """validation"""
        try:
            opportunity_id = int(self.cleaned_data["opportunity"])
            return models.Opportunity.objects.get(id=opportunity_id)
        except (ValueError, models.Opportunity.DoesNotExist):
            raise ValidationError(ugettext(u"The opportunity does'nt exist"))


class CloneActionForm(forms.Form):
    """form for clone_action: choose which type to chooses"""
    action_type = forms.ChoiceField(required=True, choices=[], label=_('New type'))

    def __init__(self, action_type, *args, **kwargs):
        super(CloneActionForm, self).__init__(*args, **kwargs)

        choices = [
            (action_type.id, action_type.name)
            for action_type in action_type.next_action_types.all().order_by('order_index')
        ]
        self.fields['action_type'].choices = choices
        # If only 1 choice = change it to a confirmation
        if len(choices) == 1:
            self.fields['action_type'].initial = choices[0][0]
            self.fields['action_type'].widget = forms.HiddenInput()
            self.single_choice = True
            self.action_type_name = choices[0][1]
        else:
            self.single_choice = False
            self.action_type_name = ''

    def clean_action_type(self):
        action_type = self.cleaned_data['action_type']
        try:
            return models.ActionType.objects.get(id=action_type)
        except models.ActionType.DoesNotExist:
            raise forms.ValidationError(_("Invalid type"))


class UpdateActionStatusForm(BsPopupModelForm):
    """form changing the status"""

    class Meta:
        """form from model"""
        model = models.Action
        fields = (
            'status', 'status2',
        )

    def __init__(self, *args, **kwargs):
        instance = kwargs.get('instance', None)
        super(UpdateActionStatusForm, self).__init__(*args, **kwargs)

        if instance and instance.id and instance.type and instance.type.allowed_status.count():
            # let javascript disable the blank value if default_status
            allowed_status = instance.type.allowed_status.all()
            if instance.frozen:
                allowed_status = allowed_status.filter(allowed_on_frozen=True)
            self.fields['status'].choices = [
                (status.id, status.name) for status in allowed_status
            ]

        if instance and instance.id and instance.type and instance.type.allowed_status2.count():
            # let javascript disable the blank value if default_status2
            allowed_status2 = instance.type.allowed_status2.all()
            if instance.frozen:
                allowed_status2 = allowed_status2.filter(allowed_on_frozen=True)
            self.fields['status2'].choices = [
                (status.id, status.name) for status in allowed_status2
            ]
        else:
            self.fields['status2'].widget = forms.HiddenInput()

    def clean_status(self):
        """status validation"""
        status = self.cleaned_data['status']
        action_type = self.instance.type
        allowed_status = ([] if action_type.default_status else [None]) + list(action_type.allowed_status.all())
        if len(allowed_status) > 0 and status not in allowed_status:
            raise ValidationError(ugettext("This status can't not be used for this action type"))
        return status

    def clean_status2(self):
        """status validation"""
        status = self.cleaned_data['status2']
        action_type = self.instance.type
        allowed_status = ([] if action_type.default_status2 else [None]) + list(action_type.allowed_status2.all())
        if len(allowed_status) > 0 and status not in allowed_status:
            raise ValidationError(ugettext("This status can't not be used for this action type"))
        return status


class UpdateActionStatus2Form(BsPopupModelForm):
    """form changing the status"""

    class Meta:
        """form from model"""
        model = models.Action
        fields = (
            'status2',
        )

    def __init__(self, *args, **kwargs):
        instance = kwargs.get('instance', None)
        super(UpdateActionStatus2Form, self).__init__(*args, **kwargs)

        if instance and instance.id and instance.type and instance.type.allowed_status2.count():
            # let javascript disable the blank value if default_status2
            self.fields['status2'].choices = [
                (status.id, status.name) for status in instance.type.allowed_status2.all()
            ]

    def clean_status2(self):
        """status validation"""
        status = self.cleaned_data['status2']
        action_type = self.instance.type
        allowed_status = ([] if action_type.default_status2 else [None]) + list(action_type.allowed_status2.all())
        if len(allowed_status) > 0 and status not in allowed_status:
            raise ValidationError(ugettext("This status can't not be used for this action type"))
        return status


class ActionMenuAdminForm(forms.ModelForm):
    """form limit the status if instance is selected"""

    def __init__(self, *args, **kwargs):
        """constructor"""
        super(ActionMenuAdminForm, self).__init__(*args, **kwargs)

        instance = kwargs.get('instance', None)

        if instance and instance.action_type and instance.action_type.allowed_status.count():
            # limit choice of status to the allowed status of the action type
            self.fields['only_for_status'].queryset = instance.action_type.allowed_status.all()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0001_initial'),
        ('coop_cms', '0002_auto_20160108_1628'),
    ]

    operations = [
        migrations.CreateModel(
            name='Emailing',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('status', models.IntegerField(default=1, choices=[(1, 'Edition in progress'), (2, 'Sending is scheduled'), (3, 'Sending in progress'), (4, 'Sent')])),
                ('scheduling_dt', models.DateTimeField(default=None, null=True, verbose_name='scheduling date', blank=True)),
                ('sending_dt', models.DateTimeField(default=None, null=True, verbose_name='sending date', blank=True)),
                ('lang', models.CharField(default=b'', max_length=5, verbose_name='language', blank=True, choices=[(b'', 'Par d\xe9faut'), (b'fr', 'Fran\xe7ais'), (b'en', 'English')])),
                ('from_email', models.CharField(default=b'', max_length=100, verbose_name='From email', blank=True)),
                ('hard_bounce', models.ManyToManyField(related_name='emailing_hard_bounce', to='Crm.Contact', blank=True)),
                ('newsletter', models.ForeignKey(to='coop_cms.Newsletter')),
                ('opened_emails', models.ManyToManyField(related_name='emailing_opened', to='Crm.Contact', blank=True)),
                ('rejected', models.ManyToManyField(related_name='emailing_rejected', to='Crm.Contact', blank=True)),
                ('send_to', models.ManyToManyField(related_name='emailing_to_be_received', to='Crm.Contact', blank=True)),
                ('sent_to', models.ManyToManyField(related_name='emailing_received', to='Crm.Contact', blank=True)),
                ('soft_bounce', models.ManyToManyField(related_name='emailing_soft_bounce', to='Crm.Contact', blank=True)),
                ('spam', models.ManyToManyField(related_name='emailing_spam', to='Crm.Contact', blank=True)),
                ('subscription_type', models.ForeignKey(to='Crm.SubscriptionType')),
                ('unsub', models.ManyToManyField(related_name='emailing_unsub', to='Crm.Contact', blank=True)),
            ],
            options={
                'verbose_name': 'emailing',
                'verbose_name_plural': 'emailings',
            },
        ),
        migrations.CreateModel(
            name='MagicLink',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('url', models.URLField(max_length=500)),
                ('uuid', models.CharField(default=b'', max_length=100, db_index=True, blank=True)),
                ('emailing', models.ForeignKey(to='Emailing.Emailing')),
                ('visitors', models.ManyToManyField(to='Crm.Contact', blank=True)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-09 11:49
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0022_auto_20180706_1344'),
    ]

    operations = [
        migrations.AddField(
            model_name='action',
            name='frozen',
            field=models.BooleanField(default=False, help_text='Some data (date, sale, ...) can not be changed ', verbose_name='frozen'),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django import template
from django.core.urlresolvers import reverse

register = template.Library()


class EmailTrackingEditNode(template.Node):
    
    def render(self, context):
        emailing = context.get('emailing', None)
        contact = context.get('contact', None)
        if emailing and contact:
            tracking_url = reverse("emailing_email_tracking", args=[emailing.id, contact.uuid])
            return '<img src="{0}" />'.format(tracking_url)
        return '<!-- tracking -->'


@register.tag
def email_tracking(parser, token):
    return EmailTrackingEditNode()


@register.filter
def dir_debug(obj):
    return '{0}'.format(dir(obj))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import balafon.utils
import balafon.Crm.models
from django.conf import settings
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Action',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('subject', models.CharField(default=b'', max_length=200, verbose_name='subject', blank=True)),
                ('planned_date', models.DateTimeField(default=None, null=True, verbose_name='planned date', db_index=True, blank=True)),
                ('detail', models.TextField(default=b'', verbose_name='detail', blank=True)),
                ('priority', models.IntegerField(default=2, verbose_name='priority', choices=[(1, 'low priority'), (2, 'medium priority'), (3, 'high priority')])),
                ('done', models.BooleanField(default=False, db_index=True, verbose_name='done')),
                ('done_date', models.DateTimeField(default=None, null=True, verbose_name='done date', db_index=True, blank=True)),
                ('display_on_board', models.BooleanField(default=True, db_index=True, verbose_name='display on board')),
                ('archived', models.BooleanField(default=False, db_index=True, verbose_name='archived')),
                ('amount', models.DecimalField(decimal_places=2, default=None, max_digits=11, blank=True, null=True, verbose_name='amount')),
                ('number', models.IntegerField(default=0, help_text='This number is auto-generated based on action type.', verbose_name='number')),
                ('end_datetime', models.DateTimeField(default=None, null=True, verbose_name='end date', db_index=True, blank=True)),
                ('uuid', models.CharField(default=b'', max_length=100, db_index=True, blank=True)),
            ],
            options={
                'verbose_name': 'action',
                'verbose_name_plural': 'actions',
            },
        ),
        migrations.CreateModel(
            name='ActionDocument',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('content', models.TextField(default=b'', verbose_name='content', blank=True)),
                ('template', models.CharField(max_length=200, verbose_name='template')),
                ('action', models.OneToOneField(to='Crm.Action')),
            ],
        ),
        migrations.CreateModel(
            name='ActionMenu',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('view_name', models.CharField(max_length=200, verbose_name='view_name')),
                ('icon', models.CharField(default=b'', max_length=30, verbose_name='icon', blank=True)),
                ('label', models.CharField(max_length=200, verbose_name='label')),
                ('a_attrs', models.CharField(default=b'', help_text='Example: class="colorbox-form" for colorbos display', max_length=50, verbose_name='Link args', blank=True)),
                ('order_index', models.IntegerField(default=0, verbose_name='order_index')),
            ],
            options={
                'ordering': ['order_index'],
                'verbose_name': 'action menu',
                'verbose_name_plural': 'action menus',
            },
        ),
        migrations.CreateModel(
            name='ActionSet',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
                ('ordering', models.IntegerField(default=10, verbose_name='display ordering')),
            ],
            options={
                'ordering': ['ordering'],
                'verbose_name': 'action set',
                'verbose_name_plural': 'action sets',
            },
        ),
        migrations.CreateModel(
            name='ActionStatus',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
                ('ordering', models.IntegerField(default=10, verbose_name='display ordering')),
            ],
            options={
                'ordering': ['ordering'],
                'verbose_name': 'action status',
                'verbose_name_plural': 'action status',
            },
        ),
        migrations.CreateModel(
            name='ActionType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
                ('subscribe_form', models.BooleanField(default=False, help_text='This action type will be proposed on the public subscribe form', verbose_name='Subscribe form')),
                ('last_number', models.IntegerField(default=0, verbose_name='last number')),
                ('number_auto_generated', models.BooleanField(default=False, verbose_name='generate number automatically')),
                ('default_template', models.CharField(default=b'', help_text='Action of this type will have a document with the given template', max_length=200, verbose_name='document template', blank=True)),
                ('is_editable', models.BooleanField(default=True, help_text='If default_template is set, define if the template has a editable content', verbose_name='is editable')),
                ('action_template', models.CharField(default=b'', help_text='Action of this type will be displayed using the given template', max_length=200, verbose_name='action template', blank=True)),
                ('order_index', models.IntegerField(default=10, verbose_name='Order')),
                ('is_amount_calculated', models.BooleanField(default=False, verbose_name='Is amount calculated')),
                ('not_assigned_when_cloned', models.BooleanField(default=False, verbose_name='Not assigned when cloned')),
                ('generate_uuid', models.BooleanField(default=False, verbose_name='Generate UUID for action')),
                ('allowed_status', models.ManyToManyField(default=None, help_text='Action of this type allow the given status', null=True, to='Crm.ActionStatus', blank=True)),
                ('default_status', models.ForeignKey(related_name='type_default_status_set', default=None, blank=True, to='Crm.ActionStatus', help_text='Default status for actions of this type', null=True)),
                ('next_action_types', models.ManyToManyField(to='Crm.ActionType', verbose_name='next action type', blank=True)),
                ('set', models.ForeignKey(default=None, blank=True, to='Crm.ActionSet', null=True, verbose_name='action set')),
            ],
            options={
                'ordering': ['order_index', 'name'],
                'verbose_name': 'action type',
                'verbose_name_plural': 'action types',
            },
        ),
        migrations.CreateModel(
            name='City',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
            ],
            options={
                'ordering': ['name'],
                'verbose_name': 'city',
                'verbose_name_plural': 'cities',
            },
        ),
        migrations.CreateModel(
            name='Contact',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('address', models.CharField(default='', max_length=200, verbose_name='address', blank=True)),
                ('address2', models.CharField(default='', max_length=200, verbose_name='address 2', blank=True)),
                ('address3', models.CharField(default='', max_length=200, verbose_name='address 3', blank=True)),
                ('zip_code', models.CharField(default='', max_length=20, verbose_name='zip code', blank=True)),
                ('cedex', models.CharField(default='', max_length=200, verbose_name='cedex', blank=True)),
                ('street_number', models.CharField(default=b'', max_length=20, verbose_name='street number', blank=True)),
                ('billing_address', models.CharField(default='', max_length=200, verbose_name='address', blank=True)),
                ('billing_address2', models.CharField(default='', max_length=200, verbose_name='address 2', blank=True)),
                ('billing_address3', models.CharField(default='', max_length=200, verbose_name='address 3', blank=True)),
                ('billing_zip_code', models.CharField(default='', max_length=20, verbose_name='zip code', blank=True)),
                ('billing_cedex', models.CharField(default='', max_length=200, verbose_name='cedex', blank=True)),
                ('billing_street_number', models.CharField(default=b'', max_length=20, verbose_name='street number', blank=True)),
                ('gender', models.IntegerField(default=0, blank=True, verbose_name='gender', choices=[(1, 'Mr'), (2, 'Mrs')])),
                ('title', models.CharField(default='', max_length=200, verbose_name='title', blank=True)),
                ('lastname', models.CharField(default='', max_length=200, verbose_name='last name', db_index=True, blank=True)),
                ('firstname', models.CharField(default='', max_length=200, verbose_name='first name', blank=True)),
                ('nickname', models.CharField(default='', max_length=200, verbose_name='nickname', blank=True)),
                ('photo', models.ImageField(default='', upload_to=balafon.Crm.models.get_contact_photo_dir, verbose_name='photo', blank=True)),
                ('birth_date', models.DateField(default=None, null=True, verbose_name='birth date', blank=True)),
                ('job', models.CharField(default='', max_length=200, verbose_name='job', blank=True)),
                ('main_contact', models.BooleanField(default=True, db_index=True, verbose_name='main contact')),
                ('accept_notifications', models.BooleanField(default=True, help_text='We may have to notify you some events (e.g. a new message).', verbose_name='accept notifications')),
                ('email_verified', models.BooleanField(default=False, verbose_name='email verified')),
                ('phone', models.CharField(default='', max_length=200, verbose_name='phone', blank=True)),
                ('mobile', models.CharField(default='', max_length=200, verbose_name='mobile', blank=True)),
                ('email', models.EmailField(default='', max_length=254, verbose_name='email', blank=True)),
                ('uuid', models.CharField(default=b'', max_length=100, db_index=True, blank=True)),
                ('notes', models.TextField(default=b'', verbose_name='notes', blank=True)),
                ('has_left', models.BooleanField(default=False, verbose_name='has left')),
                ('favorite_language', models.CharField(default=b'', max_length=10, verbose_name='favorite language', blank=True, choices=[(b'', 'Par d\xe9faut'), (b'fr', 'Fran\xe7ais'), (b'en', 'English')])),
                ('billing_city', models.ForeignKey(related_name='contact_billing_set', default=None, blank=True, to='Crm.City', null=True, verbose_name='city')),
            ],
            options={
                'ordering': ('lastname', 'firstname'),
                'verbose_name': 'contact',
                'verbose_name_plural': 'contacts',
            },
        ),
        migrations.CreateModel(
            name='ContactCustomFieldValue',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('value', models.TextField(verbose_name='value')),
                ('contact', models.ForeignKey(to='Crm.Contact')),
            ],
            options={
                'verbose_name': 'contact custom field value',
                'verbose_name_plural': 'contact custom field values',
            },
        ),
        migrations.CreateModel(
            name='ContactsImport',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('import_file', models.FileField(help_text='CSV file following the import contacts format.', upload_to=balafon.Crm.models._get_import_dir, verbose_name='import file')),
                ('name', models.CharField(default='', help_text='Optional name for searching contacts more easily. If not defined, use the name of the file.', max_length=100, verbose_name='name', blank=True)),
                ('encoding', models.CharField(default=b'utf-8', max_length=50, choices=[(b'utf-8', b'utf-8'), (b'iso-8859-15', b'iso-8859-15'), (b'cp1252', b'cp1252')])),
                ('separator', models.CharField(default=b',', max_length=5, choices=[(b',', 'Coma'), (b';', 'Semi-colon')])),
                ('entity_name_from_email', models.BooleanField(default=True, verbose_name='generate entity name from email address')),
            ],
            options={
                'verbose_name': 'contact import',
                'verbose_name_plural': 'contact imports',
            },
        ),
        migrations.CreateModel(
            name='CustomField',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                ('label', models.CharField(default=b'', max_length=100, verbose_name='label', blank=True)),
                ('model', models.IntegerField(verbose_name='model', choices=[(1, 'Entity'), (2, 'Contact')])),
                ('widget', models.CharField(default=b'', max_length=100, verbose_name='widget', blank=True)),
                ('ordering', models.IntegerField(default=10, verbose_name='display ordering')),
                ('import_order', models.IntegerField(default=0, verbose_name='import ordering')),
                ('export_order', models.IntegerField(default=0, verbose_name='export ordering')),
                ('is_link', models.BooleanField(default=False, verbose_name='is link')),
            ],
            options={
                'ordering': ('ordering', 'import_order', 'export_order'),
                'verbose_name': 'custom field',
                'verbose_name_plural': 'custom fields',
            },
        ),
        migrations.CreateModel(
            name='Entity',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('address', models.CharField(default='', max_length=200, verbose_name='address', blank=True)),
                ('address2', models.CharField(default='', max_length=200, verbose_name='address 2', blank=True)),
                ('address3', models.CharField(default='', max_length=200, verbose_name='address 3', blank=True)),
                ('zip_code', models.CharField(default='', max_length=20, verbose_name='zip code', blank=True)),
                ('cedex', models.CharField(default='', max_length=200, verbose_name='cedex', blank=True)),
                ('street_number', models.CharField(default=b'', max_length=20, verbose_name='street number', blank=True)),
                ('billing_address', models.CharField(default='', max_length=200, verbose_name='address', blank=True)),
                ('billing_address2', models.CharField(default='', max_length=200, verbose_name='address 2', blank=True)),
                ('billing_address3', models.CharField(default='', max_length=200, verbose_name='address 3', blank=True)),
                ('billing_zip_code', models.CharField(default='', max_length=20, verbose_name='zip code', blank=True)),
                ('billing_cedex', models.CharField(default='', max_length=200, verbose_name='cedex', blank=True)),
                ('billing_street_number', models.CharField(default=b'', max_length=20, verbose_name='street number', blank=True)),
                ('name', models.CharField(max_length=200, verbose_name='name', db_index=True)),
                ('description', models.CharField(default=b'', max_length=200, verbose_name='description', blank=True)),
                ('relationship_date', models.DateField(default=None, null=True, verbose_name='relationship date', blank=True)),
                ('logo', models.ImageField(default='', upload_to=balafon.Crm.models.get_entity_logo_dir, verbose_name='logo', blank=True)),
                ('phone', models.CharField(default='', max_length=200, verbose_name='phone', blank=True)),
                ('fax', models.CharField(default='', max_length=200, verbose_name='fax', blank=True)),
                ('email', models.EmailField(default='', max_length=254, verbose_name='email', blank=True)),
                ('website', models.CharField(default=b'', max_length=200, verbose_name='web site', blank=True)),
                ('notes', models.TextField(default=b'', verbose_name='notes', blank=True)),
                ('is_single_contact', models.BooleanField(default=False, verbose_name='is single contact')),
                ('billing_city', models.ForeignKey(related_name='entity_billing_set', default=None, blank=True, to='Crm.City', null=True, verbose_name='city')),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'entity',
                'verbose_name_plural': 'entities',
            },
        ),
        migrations.CreateModel(
            name='EntityCustomFieldValue',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('value', models.TextField(verbose_name='value')),
                ('custom_field', models.ForeignKey(verbose_name='custom field', to='Crm.CustomField')),
                ('entity', models.ForeignKey(to='Crm.Entity')),
            ],
            options={
                'verbose_name': 'entity custom field value',
                'verbose_name_plural': 'entity custom field values',
            },
        ),
        migrations.CreateModel(
            name='EntityRole',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
            ],
            options={
                'verbose_name': 'entity role',
                'verbose_name_plural': 'entity roles',
            },
        ),
        migrations.CreateModel(
            name='EntityType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
                ('gender', models.IntegerField(default=1, verbose_name='gender', choices=[(1, 'Male'), (2, 'Female')])),
                ('order', models.IntegerField(default=0, verbose_name='order')),
                ('logo', models.ImageField(default='', upload_to=balafon.Crm.models._get_logo_dir, verbose_name='logo', blank=True)),
                ('subscribe_form', models.BooleanField(default=True, help_text='This type will be proposed on the public subscribe form', verbose_name='Subscribe form')),
            ],
            options={
                'ordering': ['order'],
                'verbose_name': 'entity type',
                'verbose_name_plural': 'entity types',
            },
        ),
        migrations.CreateModel(
            name='Group',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('name', models.CharField(unique=True, max_length=200, verbose_name='name', db_index=True)),
                ('description', models.CharField(default=b'', max_length=200, verbose_name='description', blank=True)),
                ('subscribe_form', models.BooleanField(default=False, help_text='This group will be proposed on the public subscribe form', verbose_name='Subscribe form')),
                ('fore_color', models.CharField(default=b'', validators=[balafon.utils.validate_rgb], max_length=7, blank=True, help_text='Fore color. Must be a rgb code. For example: #ffffff', verbose_name='Fore color')),
                ('background_color', models.CharField(default=b'', validators=[balafon.utils.validate_rgb], max_length=7, blank=True, help_text='Background color. Must be a rgb code. For example: #000000', verbose_name='Background color')),
                ('contacts', models.ManyToManyField(to='Crm.Contact', null=True, blank=True)),
                ('entities', models.ManyToManyField(to='Crm.Entity', null=True, blank=True)),
            ],
            options={
                'verbose_name': 'group',
                'verbose_name_plural': 'groups',
            },
        ),
        migrations.CreateModel(
            name='Opportunity',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('name', models.CharField(max_length=200, verbose_name='name')),
                ('detail', models.TextField(default=b'', verbose_name='detail', blank=True)),
                ('probability', models.IntegerField(default=2, verbose_name='probability', choices=[(1, 'low'), (2, 'medium'), (3, 'high')])),
                ('amount', models.DecimalField(decimal_places=2, default=None, max_digits=11, blank=True, null=True, verbose_name='amount')),
                ('ended', models.BooleanField(default=False, db_index=True, verbose_name='closed')),
                ('start_date', models.DateField(default=None, null=True, verbose_name='starting date', blank=True)),
                ('end_date', models.DateField(default=None, null=True, verbose_name='closing date', blank=True)),
                ('display_on_board', models.BooleanField(default=True, db_index=True, verbose_name='display on board')),
                ('entity', models.ForeignKey(default=None, blank=True, to='Crm.Entity', null=True)),
            ],
            options={
                'verbose_name': 'opportunity',
                'verbose_name_plural': 'opportunities',
            },
        ),
        migrations.CreateModel(
            name='OpportunityStatus',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
                ('ordering', models.IntegerField(default=0)),
            ],
            options={
                'verbose_name': 'opportunity status',
                'verbose_name_plural': 'opportunity status',
            },
        ),
        migrations.CreateModel(
            name='OpportunityType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
            ],
            options={
                'verbose_name': 'opportunity type',
                'verbose_name_plural': 'opportunity types',
            },
        ),
        migrations.CreateModel(
            name='Relationship',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('contact1', models.ForeignKey(related_name='relationships1', verbose_name='contact 1', to='Crm.Contact')),
                ('contact2', models.ForeignKey(related_name='relationships2', verbose_name='contact 2', to='Crm.Contact')),
            ],
            options={
                'verbose_name': 'relationship',
                'verbose_name_plural': 'relationships',
            },
        ),
        migrations.CreateModel(
            name='RelationshipType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                ('reverse', models.CharField(default=b'', max_length=100, verbose_name='reverse relation', blank=True)),
            ],
            options={
                'verbose_name': 'relationship type',
                'verbose_name_plural': 'relationship types',
            },
        ),
        migrations.CreateModel(
            name='SameAs',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('main_contact', models.ForeignKey(default=None, blank=True, to='Crm.Contact', null=True)),
            ],
            options={
                'verbose_name': 'same as',
                'verbose_name_plural': 'sames as',
            },
        ),
        migrations.CreateModel(
            name='StreetType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'street type',
                'verbose_name_plural': 'street types',
            },
        ),
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('accept_subscription', models.BooleanField(default=False, help_text='Keep this checked if you want to receive our newsletter.', verbose_name='accept subscription')),
                ('subscription_date', models.DateTimeField(default=None, null=True, blank=True)),
                ('unsubscription_date', models.DateTimeField(default=None, null=True, blank=True)),
                ('contact', models.ForeignKey(to='Crm.Contact')),
            ],
        ),
        migrations.CreateModel(
            name='SubscriptionType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                ('site', models.ForeignKey(blank=True, to='sites.Site', null=True)),
            ],
            options={
                'verbose_name': 'Subscription type',
                'verbose_name_plural': 'Subscription types',
            },
        ),
        migrations.CreateModel(
            name='TeamMember',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                ('active', models.BooleanField(default=True, verbose_name='active')),
                ('user', models.OneToOneField(null=True, default=None, to=settings.AUTH_USER_MODEL, blank=True, verbose_name='user')),
            ],
            options={
                'verbose_name': 'team member',
                'verbose_name_plural': 'team members',
            },
        ),
        migrations.CreateModel(
            name='Zone',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
                ('code', models.CharField(default=b'', max_length=10, verbose_name='code', blank=True)),
                ('parent', models.ForeignKey(default=None, blank=True, to='Crm.Zone', null=True)),
            ],
            options={
                'ordering': ['name'],
                'verbose_name': 'zone',
                'verbose_name_plural': 'zones',
            },
        ),
        migrations.CreateModel(
            name='ZoneType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
                ('type', models.CharField(max_length=200, verbose_name='type')),
            ],
            options={
                'verbose_name': 'zone type',
                'verbose_name_plural': 'zone types',
            },
        ),
        migrations.AddField(
            model_name='zone',
            name='type',
            field=models.ForeignKey(to='Crm.ZoneType'),
        ),
        migrations.AddField(
            model_name='subscription',
            name='subscription_type',
            field=models.ForeignKey(to='Crm.SubscriptionType'),
        ),
        migrations.AddField(
            model_name='relationship',
            name='relationship_type',
            field=models.ForeignKey(verbose_name='relationship type', to='Crm.RelationshipType'),
        ),
        migrations.AddField(
            model_name='opportunity',
            name='status',
            field=models.ForeignKey(default=None, blank=True, to='Crm.OpportunityStatus', null=True),
        ),
        migrations.AddField(
            model_name='opportunity',
            name='type',
            field=models.ForeignKey(default=None, blank=True, to='Crm.OpportunityType', null=True),
        ),
        migrations.AddField(
            model_name='entity',
            name='billing_street_type',
            field=models.ForeignKey(related_name='entity_billing_set', default=None, blank=True, to='Crm.StreetType', null=True, verbose_name='street type'),
        ),
        migrations.AddField(
            model_name='entity',
            name='city',
            field=models.ForeignKey(default=None, blank=True, to='Crm.City', null=True, verbose_name='city'),
        ),
        migrations.AddField(
            model_name='entity',
            name='imported_by',
            field=models.ForeignKey(default=None, blank=True, to='Crm.ContactsImport', null=True),
        ),
        migrations.AddField(
            model_name='entity',
            name='last_modified_by',
            field=models.ForeignKey(default=None, blank=True, to=settings.AUTH_USER_MODEL, null=True, verbose_name='last modified by'),
        ),
        migrations.AddField(
            model_name='entity',
            name='street_type',
            field=models.ForeignKey(default=None, blank=True, to='Crm.StreetType', null=True, verbose_name='street type'),
        ),
        migrations.AddField(
            model_name='entity',
            name='type',
            field=models.ForeignKey(default=None, blank=True, to='Crm.EntityType', null=True, verbose_name='type'),
        ),
        migrations.AddField(
            model_name='contactsimport',
            name='entity_type',
            field=models.ForeignKey(default=None, to='Crm.EntityType', blank=True, help_text='All created entities will get this type. Ignored if the entity already exist.', null=True, verbose_name='entity type'),
        ),
        migrations.AddField(
            model_name='contactsimport',
            name='groups',
            field=models.ManyToManyField(default=None, to='Crm.Group', blank=True, help_text='The created entities will be added to the selected groups.', null=True, verbose_name='groups'),
        ),
        migrations.AddField(
            model_name='contactsimport',
            name='imported_by',
            field=models.ForeignKey(verbose_name='imported by', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='contactcustomfieldvalue',
            name='custom_field',
            field=models.ForeignKey(verbose_name='custom field', to='Crm.CustomField'),
        ),
        migrations.AddField(
            model_name='contact',
            name='billing_street_type',
            field=models.ForeignKey(related_name='contact_billing_set', default=None, blank=True, to='Crm.StreetType', null=True, verbose_name='street type'),
        ),
        migrations.AddField(
            model_name='contact',
            name='city',
            field=models.ForeignKey(default=None, blank=True, to='Crm.City', null=True, verbose_name='city'),
        ),
        migrations.AddField(
            model_name='contact',
            name='entity',
            field=models.ForeignKey(to='Crm.Entity'),
        ),
        migrations.AddField(
            model_name='contact',
            name='imported_by',
            field=models.ForeignKey(default=None, blank=True, to='Crm.ContactsImport', null=True),
        ),
        migrations.AddField(
            model_name='contact',
            name='last_modified_by',
            field=models.ForeignKey(default=None, blank=True, to=settings.AUTH_USER_MODEL, null=True, verbose_name='last modified by'),
        ),
        migrations.AddField(
            model_name='contact',
            name='relationships',
            field=models.ManyToManyField(default=None, to='Crm.Contact', null=True, through='Crm.Relationship', blank=True),
        ),
        migrations.AddField(
            model_name='contact',
            name='role',
            field=models.ManyToManyField(default=None, to='Crm.EntityRole', null=True, verbose_name='Roles', blank=True),
        ),
        migrations.AddField(
            model_name='contact',
            name='same_as',
            field=models.ForeignKey(default=None, blank=True, to='Crm.SameAs', null=True),
        ),
        migrations.AddField(
            model_name='contact',
            name='street_type',
            field=models.ForeignKey(default=None, blank=True, to='Crm.StreetType', null=True, verbose_name='street type'),
        ),
        migrations.AddField(
            model_name='city',
            name='groups',
            field=models.ManyToManyField(related_name='city_groups_set', null=True, verbose_name='group', to='Crm.Zone', blank=True),
        ),
        migrations.AddField(
            model_name='city',
            name='parent',
            field=models.ForeignKey(default=None, blank=True, to='Crm.Zone', null=True),
        ),
        migrations.AddField(
            model_name='actionmenu',
            name='action_type',
            field=models.ForeignKey(verbose_name='Action type', to='Crm.ActionType'),
        ),
        migrations.AddField(
            model_name='actionmenu',
            name='only_for_status',
            field=models.ManyToManyField(to='Crm.ActionStatus', blank=True),
        ),
        migrations.AddField(
            model_name='action',
            name='contacts',
            field=models.ManyToManyField(default=None, to='Crm.Contact', null=True, verbose_name='contacts', blank=True),
        ),
        migrations.AddField(
            model_name='action',
            name='entities',
            field=models.ManyToManyField(default=None, to='Crm.Entity', null=True, verbose_name='entities', blank=True),
        ),
        migrations.AddField(
            model_name='action',
            name='in_charge',
            field=models.ForeignKey(default=None, blank=True, to='Crm.TeamMember', null=True, verbose_name='in charge'),
        ),
        migrations.AddField(
            model_name='action',
            name='last_modified_by',
            field=models.ForeignKey(default=None, blank=True, to=settings.AUTH_USER_MODEL, null=True, verbose_name='last modified by'),
        ),
        migrations.AddField(
            model_name='action',
            name='opportunity',
            field=models.ForeignKey(default=None, blank=True, to='Crm.Opportunity', null=True, verbose_name='opportunity'),
        ),
        migrations.AddField(
            model_name='action',
            name='parent',
            field=models.ForeignKey(default=None, blank=True, to='Crm.Action', null=True, verbose_name='parent'),
        ),
        migrations.AddField(
            model_name='action',
            name='status',
            field=models.ForeignKey(default=None, blank=True, to='Crm.ActionStatus', null=True),
        ),
        migrations.AddField(
            model_name='action',
            name='type',
            field=models.ForeignKey(default=None, blank=True, to='Crm.ActionType', null=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0005_auto_20160129_1559'),
    ]

    operations = [
        migrations.AlterField(
            model_name='contact',
            name='gender',
            field=models.IntegerField(default=0, blank=True, verbose_name='gender', choices=[(1, 'Mr'), (2, 'Mrs'), (3, 'Mrs and Mr')]),
        ),
    ]



# -*- coding: utf-8 -*-
"""unit testing"""

from __future__ import unicode_literals

from datetime import datetime
from decimal import Decimal
import logging
from unittest import skipIf

from django.conf import settings
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.core import mail
from django.test.utils import override_settings
from django.utils.translation import ugettext as _

from model_mommy import mommy
from rest_framework import status
from rest_framework.test import APITestCase, APIClient

from balafon.Crm.models import Action, ActionType
from balafon.Profile.utils import create_profile_contact
from balafon.Store import models
from balafon.Store.settings import get_cart_type_name


def custom_confirmation_email_subject(profile, action):
    return 'Hello {0}'.format(''.join([contact.firstname for contact in action.contacts.all()]))


GLOBAL_COUNTER = 0


def cart_processed_callback(cart):
    global GLOBAL_COUNTER
    GLOBAL_COUNTER += 1


class BaseTestCase(APITestCase):
    """Base class for test cases"""

    def setUp(self):
        logging.disable(logging.CRITICAL)
        self.user = User.objects.create(username="toto", is_active=True, is_staff=False, email='toto@toto.fr')
        self.user.set_password("abc")
        self.user.save()
        self.client = APIClient()
        self._login()

    def tearDown(self):
        logging.disable(logging.NOTSET)

    def _login(self):
        self.client.login(username="toto", password="abc")


@skipIf(not ("balafon.Profile" in settings.INSTALLED_APPS), "registration not installed")
class CartTest(BaseTestCase):
    """Test that cart is process properly"""

    def test_post_cart(self):
        """It should create a new sale and action"""

        # Create contact for the user
        profile = create_profile_contact(self.user)
        contact = profile.contact

        store_item1 = mommy.make(models.StoreItem)
        store_item2 = mommy.make(models.StoreItem)

        delivery_point = mommy.make(models.DeliveryPoint)

        data = {
            'items': [
                {'id': store_item1.id, 'quantity': 2},
                {'id': store_item2.id, 'quantity': 1},
            ],
            'delivery_point': delivery_point.id,
            'purchase_datetime': datetime(2015, 7, 23, 12, 0)
        }

        url = reverse('store_post_cart')

        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(response.data['ok'], True)
        self.assertEqual(len(response.data['warnings']), 0)
        self.assertEqual(response.data['deliveryDate'], data['purchase_datetime'])
        self.assertEqual(response.data['deliveryPlace'], delivery_point.name)

        action_type = ActionType.objects.get(name=get_cart_type_name())

        action_queryset = Action.objects.filter(type=action_type)
        self.assertEqual(action_queryset.count(), 1)
        action = action_queryset[0]

        self.assertEqual(list(action.contacts.all()), [contact])
        self.assertEqual(action.subject, '')
        self.assertEqual(action.detail, '')
        self.assertEqual(action.planned_date, data['purchase_datetime'])

        self.assertEqual(action.sale.analysis_code, models.SaleAnalysisCode.objects.get(name='Internet'))

        self.assertEqual(action.sale.saleitem_set.count(), 2)
        self.assertEqual(action.sale.saleitem_set.all()[0].item, store_item1)
        self.assertEqual(action.sale.saleitem_set.all()[0].text, store_item1.name)
        self.assertEqual(action.sale.saleitem_set.all()[0].unit_price(), store_item1.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[0].vat_rate, store_item1.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[0].quantity, 2)

        self.assertEqual(action.sale.saleitem_set.all()[1].item, store_item2)
        self.assertEqual(action.sale.saleitem_set.all()[1].text, store_item2.name)
        self.assertEqual(action.sale.saleitem_set.all()[1].unit_price(), store_item2.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[1].vat_rate, store_item2.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[1].quantity, 1)

        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(mail.outbox[0].to, [profile.contact.email])
        self.assertEqual(mail.outbox[1].to, [settings.BALAFON_NOTIFICATION_EMAIL])

        store_item1 = models.StoreItem.objects.get(id=store_item1.id)
        self.assertEqual(store_item1.stock_count, None)

        store_item2 = models.StoreItem.objects.get(id=store_item2.id)
        self.assertEqual(store_item2.stock_count, None)

    @override_settings(BALAFON_CART_CONFIRMATION_EMAIL_SUBJECT="Hello")
    def test_post_cart_email_subject(self):
        """It should create a new sale and action"""

        # Create contact for the user
        profile = create_profile_contact(self.user)
        contact = profile.contact

        store_item1 = mommy.make(models.StoreItem)
        store_item2 = mommy.make(models.StoreItem)

        delivery_point = mommy.make(models.DeliveryPoint)

        data = {
            'items': [
                {'id': store_item1.id, 'quantity': 2},
                {'id': store_item2.id, 'quantity': 1},
            ],
            'delivery_point': delivery_point.id,
            'purchase_datetime': datetime(2015, 7, 23, 12, 0)
        }

        url = reverse('store_post_cart')

        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(response.data['ok'], True)
        self.assertEqual(len(response.data['warnings']), 0)
        self.assertEqual(response.data['deliveryDate'], data['purchase_datetime'])
        self.assertEqual(response.data['deliveryPlace'], delivery_point.name)

        action_type = ActionType.objects.get(name=get_cart_type_name())

        action_queryset = Action.objects.filter(type=action_type)
        self.assertEqual(action_queryset.count(), 1)
        action = action_queryset[0]

        self.assertEqual(list(action.contacts.all()), [contact])
        self.assertEqual(action.subject, '')
        self.assertEqual(action.detail, '')
        self.assertEqual(action.planned_date, data['purchase_datetime'])

        self.assertEqual(action.sale.analysis_code, models.SaleAnalysisCode.objects.get(name='Internet'))

        self.assertEqual(action.sale.saleitem_set.count(), 2)
        self.assertEqual(action.sale.saleitem_set.all()[0].item, store_item1)
        self.assertEqual(action.sale.saleitem_set.all()[0].text, store_item1.name)
        self.assertEqual(action.sale.saleitem_set.all()[0].unit_price(), store_item1.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[0].vat_rate, store_item1.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[0].quantity, 2)

        self.assertEqual(action.sale.saleitem_set.all()[1].item, store_item2)
        self.assertEqual(action.sale.saleitem_set.all()[1].text, store_item2.name)
        self.assertEqual(action.sale.saleitem_set.all()[1].unit_price(), store_item2.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[1].vat_rate, store_item2.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[1].quantity, 1)

        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(mail.outbox[0].to, [profile.contact.email])
        self.assertEqual(mail.outbox[0].subject, "Hello")
        self.assertEqual(mail.outbox[1].to, [settings.BALAFON_NOTIFICATION_EMAIL])

        store_item1 = models.StoreItem.objects.get(id=store_item1.id)
        self.assertEqual(store_item1.stock_count, None)

        store_item2 = models.StoreItem.objects.get(id=store_item2.id)
        self.assertEqual(store_item2.stock_count, None)

    @override_settings(BALAFON_CART_CONFIRMATION_EMAIL_SUBJECT=custom_confirmation_email_subject)
    def test_post_cart_email_custom_subject(self):
        """It should create a new sale and action"""

        # Create contact for the user
        profile = create_profile_contact(self.user)
        contact = profile.contact
        contact.firstname = "Bob"
        contact.save()

        store_item1 = mommy.make(models.StoreItem)
        store_item2 = mommy.make(models.StoreItem)

        delivery_point = mommy.make(models.DeliveryPoint)

        data = {
            'items': [
                {'id': store_item1.id, 'quantity': 2},
                {'id': store_item2.id, 'quantity': 1},
            ],
            'delivery_point': delivery_point.id,
            'purchase_datetime': datetime(2015, 7, 23, 12, 0)
        }

        url = reverse('store_post_cart')

        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(response.data['ok'], True)
        self.assertEqual(len(response.data['warnings']), 0)
        self.assertEqual(response.data['deliveryDate'], data['purchase_datetime'])
        self.assertEqual(response.data['deliveryPlace'], delivery_point.name)

        action_type = ActionType.objects.get(name=get_cart_type_name())

        action_queryset = Action.objects.filter(type=action_type)
        self.assertEqual(action_queryset.count(), 1)
        action = action_queryset[0]

        self.assertEqual(list(action.contacts.all()), [contact])
        self.assertEqual(action.subject, '')
        self.assertEqual(action.detail, '')
        self.assertEqual(action.planned_date, data['purchase_datetime'])

        self.assertEqual(action.sale.analysis_code, models.SaleAnalysisCode.objects.get(name='Internet'))

        self.assertEqual(action.sale.saleitem_set.count(), 2)
        self.assertEqual(action.sale.saleitem_set.all()[0].item, store_item1)
        self.assertEqual(action.sale.saleitem_set.all()[0].text, store_item1.name)
        self.assertEqual(action.sale.saleitem_set.all()[0].unit_price(), store_item1.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[0].vat_rate, store_item1.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[0].quantity, 2)

        self.assertEqual(action.sale.saleitem_set.all()[1].item, store_item2)
        self.assertEqual(action.sale.saleitem_set.all()[1].text, store_item2.name)
        self.assertEqual(action.sale.saleitem_set.all()[1].unit_price(), store_item2.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[1].vat_rate, store_item2.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[1].quantity, 1)

        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(mail.outbox[0].to, [profile.contact.email])
        self.assertEqual(mail.outbox[0].subject, "Hello Bob")
        self.assertEqual(mail.outbox[1].to, [settings.BALAFON_NOTIFICATION_EMAIL])

        store_item1 = models.StoreItem.objects.get(id=store_item1.id)
        self.assertEqual(store_item1.stock_count, None)

        store_item2 = models.StoreItem.objects.get(id=store_item2.id)
        self.assertEqual(store_item2.stock_count, None)

    def test_post_cart_notes(self):
        """It should create a new sale and action"""

        # Create contact for the user
        profile = create_profile_contact(self.user)
        contact = profile.contact

        store_item1 = mommy.make(models.StoreItem)
        store_item2 = mommy.make(models.StoreItem)

        delivery_point = mommy.make(models.DeliveryPoint)

        data = {
            'items': [
                {'id': store_item1.id, 'quantity': 2},
                {'id': store_item2.id, 'quantity': 1},
            ],
            'delivery_point': delivery_point.id,
            'notes': 'This is a text',
            'purchase_datetime': datetime(2015, 7, 23, 12, 0)
        }

        url = reverse('store_post_cart')

        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(response.data['ok'], True)
        self.assertEqual(len(response.data['warnings']), 0)
        self.assertEqual(response.data['deliveryDate'], data['purchase_datetime'])
        self.assertEqual(response.data['deliveryPlace'], delivery_point.name)

        action_type = ActionType.objects.get(name=get_cart_type_name())

        action_queryset = Action.objects.filter(type=action_type)
        self.assertEqual(action_queryset.count(), 1)
        action = action_queryset[0]

        self.assertEqual(list(action.contacts.all()), [contact])
        self.assertEqual(action.subject, _('Notes'))
        self.assertEqual(action.detail, data['notes'])
        self.assertEqual(action.planned_date, data['purchase_datetime'])

        self.assertEqual(action.sale.saleitem_set.count(), 2)
        self.assertEqual(action.sale.saleitem_set.all()[0].item, store_item1)
        self.assertEqual(action.sale.saleitem_set.all()[0].text, store_item1.name)
        self.assertEqual(action.sale.saleitem_set.all()[0].unit_price(), store_item1.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[0].vat_rate, store_item1.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[0].quantity, 2)

        self.assertEqual(action.sale.saleitem_set.all()[1].item, store_item2)
        self.assertEqual(action.sale.saleitem_set.all()[1].text, store_item2.name)
        self.assertEqual(action.sale.saleitem_set.all()[1].unit_price(), store_item2.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[1].vat_rate, store_item2.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[1].quantity, 1)

        self.assertEqual(len(mail.outbox), 2)

    def test_post_cart_long_notes(self):
        """It should create a new sale and action"""

        # Create contact for the user
        profile = create_profile_contact(self.user)
        contact = profile.contact

        store_item1 = mommy.make(models.StoreItem)
        store_item2 = mommy.make(models.StoreItem)

        delivery_point = mommy.make(models.DeliveryPoint)

        data = {
            'items': [
                {'id': store_item1.id, 'quantity': 2},
                {'id': store_item2.id, 'quantity': 1},
            ],
            'delivery_point': delivery_point.id,
            'notes': 'abc'*100,
            'purchase_datetime': datetime(2015, 7, 23, 12, 0)
        }

        url = reverse('store_post_cart')

        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(response.data['ok'], True)
        self.assertEqual(len(response.data['warnings']), 0)
        self.assertEqual(response.data['deliveryDate'], data['purchase_datetime'])
        self.assertEqual(response.data['deliveryPlace'], delivery_point.name)

        action_type = ActionType.objects.get(name=get_cart_type_name())

        action_queryset = Action.objects.filter(type=action_type)
        self.assertEqual(action_queryset.count(), 1)
        action = action_queryset[0]

        self.assertEqual(list(action.contacts.all()), [contact])
        self.assertEqual(action.subject, _('Notes'))
        self.assertEqual(action.detail, data['notes'])
        self.assertEqual(action.planned_date, data['purchase_datetime'])

        self.assertEqual(action.sale.saleitem_set.count(), 2)
        self.assertEqual(action.sale.saleitem_set.all()[0].item, store_item1)
        self.assertEqual(action.sale.saleitem_set.all()[0].text, store_item1.name)
        self.assertEqual(action.sale.saleitem_set.all()[0].unit_price(), store_item1.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[0].vat_rate, store_item1.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[0].quantity, 2)

        self.assertEqual(action.sale.saleitem_set.all()[1].item, store_item2)
        self.assertEqual(action.sale.saleitem_set.all()[1].text, store_item2.name)
        self.assertEqual(action.sale.saleitem_set.all()[1].unit_price(), store_item2.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[1].vat_rate, store_item2.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[1].quantity, 1)

        self.assertEqual(len(mail.outbox), 2)

    def test_post_cart_notes_several_lines(self):
        """It should create a new sale and action"""

        # Create contact for the user
        profile = create_profile_contact(self.user)
        contact = profile.contact

        store_item1 = mommy.make(models.StoreItem)
        store_item2 = mommy.make(models.StoreItem)

        delivery_point = mommy.make(models.DeliveryPoint)

        data = {
            'items': [
                {'id': store_item1.id, 'quantity': 2},
                {'id': store_item2.id, 'quantity': 1},
            ],
            'delivery_point': delivery_point.id,
            'notes': 'a\nB\nc',
            'purchase_datetime': datetime(2015, 7, 23, 12, 0)
        }

        url = reverse('store_post_cart')

        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(response.data['ok'], True)
        self.assertEqual(len(response.data['warnings']), 0)
        self.assertEqual(response.data['deliveryDate'], data['purchase_datetime'])
        self.assertEqual(response.data['deliveryPlace'], delivery_point.name)

        action_type = ActionType.objects.get(name=get_cart_type_name())

        action_queryset = Action.objects.filter(type=action_type)
        self.assertEqual(action_queryset.count(), 1)
        action = action_queryset[0]

        self.assertEqual(list(action.contacts.all()), [contact])
        self.assertEqual(action.subject, _('Notes'))
        self.assertEqual(action.detail, 'a\nB\nc')
        self.assertEqual(action.planned_date, data['purchase_datetime'])

        self.assertEqual(action.sale.saleitem_set.count(), 2)
        self.assertEqual(action.sale.saleitem_set.all()[0].item, store_item1)
        self.assertEqual(action.sale.saleitem_set.all()[0].text, store_item1.name)
        self.assertEqual(action.sale.saleitem_set.all()[0].unit_price(), store_item1.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[0].vat_rate, store_item1.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[0].quantity, 2)

        self.assertEqual(action.sale.saleitem_set.all()[1].item, store_item2)
        self.assertEqual(action.sale.saleitem_set.all()[1].text, store_item2.name)
        self.assertEqual(action.sale.saleitem_set.all()[1].unit_price(), store_item2.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[1].vat_rate, store_item2.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[1].quantity, 1)

        self.assertEqual(len(mail.outbox), 2)

    def test_post_cart_update_stock(self):
        """It should create a new sale and action and update the stock"""

        # Create contact for the user
        profile = create_profile_contact(self.user)
        contact = profile.contact

        store_item1 = mommy.make(models.StoreItem, stock_count=Decimal('10'))
        store_item2 = mommy.make(models.StoreItem, stock_count=Decimal('0.5'))

        delivery_point = mommy.make(models.DeliveryPoint)

        data = {
            'items': [
                {'id': store_item1.id, 'quantity': 2},
                {'id': store_item2.id, 'quantity': 1},
            ],
            'delivery_point': delivery_point.id,
            'purchase_datetime': datetime(2015, 7, 23, 12, 0)
        }

        url = reverse('store_post_cart')

        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(response.data['ok'], True)
        self.assertEqual(len(response.data['warnings']), 0)
        self.assertEqual(response.data['deliveryDate'], data['purchase_datetime'])
        self.assertEqual(response.data['deliveryPlace'], delivery_point.name)

        action_type = ActionType.objects.get(name=get_cart_type_name())

        action_queryset = Action.objects.filter(type=action_type)
        self.assertEqual(action_queryset.count(), 1)
        action = action_queryset[0]

        self.assertEqual(list(action.contacts.all()), [contact])
        self.assertEqual(action.subject, '')
        self.assertEqual(action.detail, '')
        self.assertEqual(action.planned_date, data['purchase_datetime'])

        self.assertEqual(action.sale.saleitem_set.count(), 2)
        self.assertEqual(action.sale.saleitem_set.all()[0].item, store_item1)
        self.assertEqual(action.sale.saleitem_set.all()[0].text, store_item1.name)
        self.assertEqual(action.sale.saleitem_set.all()[0].unit_price(), store_item1.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[0].vat_rate, store_item1.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[0].quantity, 2)

        self.assertEqual(action.sale.saleitem_set.all()[1].item, store_item2)
        self.assertEqual(action.sale.saleitem_set.all()[1].text, store_item2.name)
        self.assertEqual(action.sale.saleitem_set.all()[1].unit_price(), store_item2.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[1].vat_rate, store_item2.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[1].quantity, 1)

        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(mail.outbox[0].to, [profile.contact.email])
        self.assertEqual(mail.outbox[1].to, [settings.BALAFON_NOTIFICATION_EMAIL])

        store_item1 = models.StoreItem.objects.get(id=store_item1.id)
        self.assertEqual(store_item1.stock_count, Decimal("8"))

        store_item2 = models.StoreItem.objects.get(id=store_item2.id)
        self.assertEqual(store_item2.stock_count, Decimal("-0.5"))

    def test_post_cart_not_available(self):
        """It should create a new sale and ignore articles which are not available"""

        # Create contact for the user
        profile = create_profile_contact(self.user)
        contact = profile.contact

        store_item1 = mommy.make(models.StoreItem, stock_count=Decimal('10'))
        store_item2 = mommy.make(models.StoreItem, stock_count=Decimal('0.5'), available=False)

        delivery_point = mommy.make(models.DeliveryPoint)

        data = {
            'items': [
                {'id': store_item1.id, 'quantity': 2},
                {'id': store_item2.id, 'quantity': 1},
            ],
            'delivery_point': delivery_point.id,
            'purchase_datetime': datetime(2015, 7, 23, 12, 0)
        }

        url = reverse('store_post_cart')

        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(response.data['ok'], True)
        self.assertEqual(len(response.data['warnings']), 1)
        self.assertEqual(response.data['deliveryDate'], data['purchase_datetime'])
        self.assertEqual(response.data['deliveryPlace'], delivery_point.name)

        action_type = ActionType.objects.get(name=get_cart_type_name())

        action_queryset = Action.objects.filter(type=action_type)
        self.assertEqual(action_queryset.count(), 1)
        action = action_queryset[0]

        self.assertEqual(list(action.contacts.all()), [contact])
        self.assertEqual(action.subject, '')
        self.assertEqual(action.detail, '')
        self.assertEqual(action.planned_date, data['purchase_datetime'])

        self.assertEqual(action.sale.saleitem_set.count(), 1)
        self.assertEqual(action.sale.saleitem_set.all()[0].item, store_item1)
        self.assertEqual(action.sale.saleitem_set.all()[0].text, store_item1.name)
        self.assertEqual(action.sale.saleitem_set.all()[0].unit_price(), store_item1.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[0].vat_rate, store_item1.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[0].quantity, 2)

        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(mail.outbox[0].to, [profile.contact.email])
        self.assertEqual(mail.outbox[1].to, [settings.BALAFON_NOTIFICATION_EMAIL])

        store_item1 = models.StoreItem.objects.get(id=store_item1.id)
        self.assertEqual(store_item1.stock_count, Decimal("8"))

        store_item2 = models.StoreItem.objects.get(id=store_item2.id)
        self.assertEqual(store_item2.stock_count, Decimal("0.5"))

    def test_post_cart_empty(self):
        """It should create a new sale and ignore articles which are not available"""

        # Create contact for the user
        profile = create_profile_contact(self.user)
        contact = profile.contact

        store_item1 = mommy.make(models.StoreItem, stock_count=Decimal('10'), available=False)
        store_item2 = mommy.make(models.StoreItem, stock_count=Decimal('0.5'), available=False)

        delivery_point = mommy.make(models.DeliveryPoint)

        data = {
            'items': [
                {'id': store_item1.id, 'quantity': 2},
                {'id': store_item2.id, 'quantity': 1},
            ],
            'delivery_point': delivery_point.id,
            'purchase_datetime': datetime(2015, 7, 23, 12, 0)
        }

        url = reverse('store_post_cart')

        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(response.data['ok'], False)
        self.assertEqual(len(response.data['warnings']), 2)

        action_type = ActionType.objects.get(name=get_cart_type_name())

        action_queryset = Action.objects.filter(type=action_type)
        self.assertEqual(action_queryset.count(), 0)

        self.assertEqual(len(mail.outbox), 0)

        store_item1 = models.StoreItem.objects.get(id=store_item1.id)
        self.assertEqual(store_item1.stock_count, Decimal("10"))

        store_item2 = models.StoreItem.objects.get(id=store_item2.id)
        self.assertEqual(store_item2.stock_count, Decimal("0.5"))

    def test_cart_when_deleted_article(self):
        """The cart should not be removed if store_item is deleted"""

        # Create contact for the user
        profile = create_profile_contact(self.user)
        contact = profile.contact

        store_item1 = mommy.make(models.StoreItem, stock_count=Decimal('10'))
        store_item2 = mommy.make(models.StoreItem, stock_count=Decimal('0.5'))

        sale = mommy.make(models.Sale)

        sale_item1 = mommy.make(
            models.SaleItem,
            sale=sale,
            item=store_item1,
            quantity=2,
            vat_rate=store_item1.vat_rate,
            pre_tax_price=store_item1.pre_tax_price
        )

        sale_item2 = mommy.make(
            models.SaleItem,
            sale=sale,
            item=store_item2,
            quantity=1,
            vat_rate=store_item2.vat_rate,
            pre_tax_price=store_item2.pre_tax_price
        )

        store_item1.delete()

        self.assertEqual(1, models.Sale.objects.count())
        self.assertEqual(sale.id, models.Sale.objects.all()[0].id)

    def test_cart_with_discount(self):
        """The cart should take account discount into account"""

        # Create contact for the user
        profile = create_profile_contact(self.user)
        contact = profile.contact

        discount = mommy.make(
            models.Discount, active=True, name="10% si + de 5", rate=Decimal("10"), quantity=Decimal("5")
        )
        price_class = mommy.make(models.PriceClass, name="Vrac")
        price_class.discounts.add(discount)
        price_class.save()

        store_item1 = mommy.make(models.StoreItem, pre_tax_price=Decimal('10'), price_class=price_class)
        store_item2 = mommy.make(models.StoreItem, pre_tax_price=Decimal('1'), price_class=price_class)
        store_item3 = mommy.make(models.StoreItem, pre_tax_price=Decimal('2'), price_class=None)

        delivery_point = mommy.make(models.DeliveryPoint)

        data = {
            'items': [
                {'id': store_item1.id, 'quantity': 5},
                {'id': store_item2.id, 'quantity': 2},
                {'id': store_item3.id, 'quantity': 5},
            ],
            'delivery_point': delivery_point.id,
            'purchase_datetime': datetime(2015, 7, 23, 12, 0)
        }

        url = reverse('store_post_cart')

        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(response.data['ok'], True)
        self.assertEqual(len(response.data['warnings']), 0)

        action_type = ActionType.objects.get(name=get_cart_type_name())

        action_queryset = Action.objects.filter(type=action_type)
        self.assertEqual(action_queryset.count(), 1)
        action = action_queryset[0]

        self.assertEqual(list(action.contacts.all()), [contact])
        self.assertEqual(action.subject, '')
        self.assertEqual(action.detail, '')
        self.assertEqual(action.planned_date, data['purchase_datetime'])

        self.assertEqual(action.sale.saleitem_set.count(), 3)
        self.assertEqual(action.sale.saleitem_set.all()[0].item, store_item1)
        self.assertEqual(action.sale.saleitem_set.all()[0].text, store_item1.name)
        self.assertEqual(action.sale.saleitem_set.all()[0].unit_price(), store_item1.pre_tax_price * Decimal("0.9"))
        self.assertEqual(action.sale.saleitem_set.all()[0].vat_rate, store_item1.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[0].quantity, 5)

        self.assertEqual(action.sale.saleitem_set.all()[1].item, store_item2)
        self.assertEqual(action.sale.saleitem_set.all()[1].text, store_item2.name)
        self.assertEqual(action.sale.saleitem_set.all()[1].unit_price(), store_item2.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[1].vat_rate, store_item2.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[1].quantity, 2)

        self.assertEqual(action.sale.saleitem_set.all()[2].item, store_item3)
        self.assertEqual(action.sale.saleitem_set.all()[2].text, store_item3.name)
        self.assertEqual(action.sale.saleitem_set.all()[2].unit_price(), store_item3.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[2].vat_rate, store_item3.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[2].quantity, 5)

        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(mail.outbox[0].to, [profile.contact.email])
        self.assertEqual(mail.outbox[1].to, [settings.BALAFON_NOTIFICATION_EMAIL])

    @override_settings(BALAFON_CART_PROCESSED_CALLBACK='balafon.Store.tests.test_cart.cart_processed_callback')
    def test_post_cart_callback(self):
        """It should create a new sale and action"""

        global GLOBAL_COUNTER
        GLOBAL_COUNTER = 0

        # Create contact for the user
        profile = create_profile_contact(self.user)
        contact = profile.contact

        store_item1 = mommy.make(models.StoreItem)
        store_item2 = mommy.make(models.StoreItem)

        delivery_point = mommy.make(models.DeliveryPoint)

        data = {
            'items': [
                {'id': store_item1.id, 'quantity': 2},
                {'id': store_item2.id, 'quantity': 1},
            ],
            'delivery_point': delivery_point.id,
            'purchase_datetime': datetime(2015, 7, 23, 12, 0)
        }

        url = reverse('store_post_cart')

        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(response.data['ok'], True)
        self.assertEqual(len(response.data['warnings']), 0)
        self.assertEqual(response.data['deliveryDate'], data['purchase_datetime'])
        self.assertEqual(response.data['deliveryPlace'], delivery_point.name)

        action_type = ActionType.objects.get(name=get_cart_type_name())

        action_queryset = Action.objects.filter(type=action_type)
        self.assertEqual(action_queryset.count(), 1)
        action = action_queryset[0]

        self.assertEqual(list(action.contacts.all()), [contact])
        self.assertEqual(action.subject, '')
        self.assertEqual(action.detail, '')
        self.assertEqual(action.planned_date, data['purchase_datetime'])

        self.assertEqual(action.sale.analysis_code, models.SaleAnalysisCode.objects.get(name='Internet'))

        self.assertEqual(action.sale.saleitem_set.count(), 2)
        self.assertEqual(action.sale.saleitem_set.all()[0].item, store_item1)
        self.assertEqual(action.sale.saleitem_set.all()[0].text, store_item1.name)
        self.assertEqual(action.sale.saleitem_set.all()[0].unit_price(), store_item1.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[0].vat_rate, store_item1.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[0].quantity, 2)

        self.assertEqual(action.sale.saleitem_set.all()[1].item, store_item2)
        self.assertEqual(action.sale.saleitem_set.all()[1].text, store_item2.name)
        self.assertEqual(action.sale.saleitem_set.all()[1].unit_price(), store_item2.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[1].vat_rate, store_item2.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[1].quantity, 1)

        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(mail.outbox[0].to, [profile.contact.email])
        self.assertEqual(mail.outbox[1].to, [settings.BALAFON_NOTIFICATION_EMAIL])

        store_item1 = models.StoreItem.objects.get(id=store_item1.id)
        self.assertEqual(store_item1.stock_count, None)

        store_item2 = models.StoreItem.objects.get(id=store_item2.id)
        self.assertEqual(store_item2.stock_count, None)

        # Make sure the calback has been called
        self.assertEqual(GLOBAL_COUNTER, 1)


# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-25 09:05
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0015_storemanagementactiontype_default_readonly_status'),
    ]

    operations = [
        migrations.AddField(
            model_name='storeitem',
            name='item_accounting_code',
            field=models.CharField(blank=True, default='', max_length=20, verbose_name='accounting code'),
        ),
        migrations.AddField(
            model_name='storeitemcategory',
            name='accounting_code',
            field=models.CharField(blank=True, default='', max_length=20, verbose_name='accounting code'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-04-09 13:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Emailing', '0002_auto_20160601_1128'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailing',
            name='from_email',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='From email'),
        ),
        migrations.AlterField(
            model_name='emailing',
            name='lang',
            field=models.CharField(blank=True, choices=[('', 'Default'), ('en', 'English'), ('fr', 'Français')], default='', max_length=5, verbose_name='language'),
        ),
        migrations.AlterField(
            model_name='magiclink',
            name='uuid',
            field=models.CharField(blank=True, db_index=True, default='', max_length=100),
        ),
    ]



# -*- coding: utf-8 -*-
"""user customization"""

from __future__ import unicode_literals

from django.conf import settings
from django.core.urlresolvers import reverse, NoReverseMatch
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.utils.encoding import python_2_unicode_compatible

from coop_cms.utils import RequestManager, RequestNotFound

from balafon.permissions import can_access


@python_2_unicode_compatible
class UserPreferences(models.Model):
    """user preferences"""

    user = models.OneToOneField(User)
    notify_due_actions = models.BooleanField(default=False, verbose_name=_("Notify due actions"))
    message_in_favorites = models.BooleanField(
        default=False,
        verbose_name=_("Create automatically a favorite for message posted from the public form")
    )

    class Meta:
        verbose_name = _('User preferences')
        verbose_name_plural = _('User preferences')
    
    def __str__(self):
        return self.user.username


@python_2_unicode_compatible
class Favorite(models.Model):
    """user favorite items"""

    user = models.ForeignKey(User, verbose_name=_("user"), related_name="user_favorite_set")
    content_type = models.ForeignKey(ContentType, verbose_name=_("content_type"), related_name="user_favorite_set")
    object_id = models.PositiveIntegerField(verbose_name=_("object id"))
    content_object = GenericForeignKey('content_type', 'object_id')
    
    class Meta:
        verbose_name = _('Favorite')
        verbose_name_plural = _('Favorites')
        unique_together = (('user', 'content_type', 'object_id'),)
        
    def __str__(self):
        return "{0} - {1}".format(self.user, self.content_object)


@python_2_unicode_compatible
class UserHomepage(models.Model):
    """User homepage"""

    user = models.OneToOneField(User, verbose_name=_("user"))
    url = models.URLField(verbose_name=_("URL"))
    
    class Meta:
        verbose_name = _('User homepage')
        verbose_name_plural = _('User homepages')
        
    def __str__(self):
        return "{0} - {1}".format(self.user, self.url)


@python_2_unicode_compatible
class CustomMenu(models.Model):
    """Menus that can be added to the Balafon menu"""
    POSITION_MENU = 0
    POSITION_PLANNING = 1

    POSITION_CHOICES = (
        (POSITION_MENU, _('Menu')),
        (POSITION_PLANNING, _('Planning')),
    )

    label = models.CharField(max_length=100, verbose_name=_("label"))
    icon = models.CharField(max_length=20, verbose_name=_("icon"), blank=True, default="")
    order_index = models.IntegerField(default=0)
    position = models.IntegerField(
        default=0, choices=POSITION_CHOICES, verbose_name=_('position'),
        help_text=_('Where the menu will be set')
    )

    class Meta:
        verbose_name = _('Custom menu')
        verbose_name_plural = _('Custom menus')
        ordering = ['order_index', 'label']

    def get_children(self):
        """returns children"""
        children = []
        try:
            request = RequestManager().get_request()
            if can_access(request.user):
                for item in self.custommenuitem_set.all():
                    if item.only_for_users.count() == 0 or (request.user in item.only_for_users.all()):
                        if item.get_url(*request.resolver_match.args, **request.resolver_match.kwargs):
                            children.append(item)
        except (RequestNotFound, AttributeError):
            pass
        return children

    def __str__(self):
        return self.label


@python_2_unicode_compatible
class CustomMenuItem(models.Model):
    """Menus items hat can be added to the Balafon menu"""
    parent = models.ForeignKey(CustomMenu)
    label = models.CharField(max_length=100, verbose_name=_("label"))
    icon = models.CharField(max_length=20, verbose_name=_("icon"), blank=True, default="")
    url = models.CharField(max_length=100, verbose_name=_("url"), default='', blank=True)
    reverse = models.CharField(max_length=100, verbose_name=_("reverse"), default='', blank=True)
    reverse_kwargs = models.CharField(
        max_length=100, verbose_name=_("reverse_kwargs"), default='', blank=True,
        help_text=_('kwargs to use for building the reverse. kw1:defaultval1,kw2,kw3:defaultval3 ')
    )
    order_index = models.IntegerField(default=0)
    only_for_users = models.ManyToManyField(
        User, blank=True, verbose_name=_("only for users"), limit_choices_to={'is_staff': True}
    )
    attributes = models.CharField(max_length=100, verbose_name=_("attributes"), default="", blank=True)

    class Meta:
        verbose_name = _('Custom menu item')
        verbose_name_plural = _('Custom menu items')
        ordering = ['order_index', 'label']

    def __str__(self):
        return "{0} > {1}".format(self.parent.label, self.label)

    def get_url(self, *args, **kwargs):
        """return the url to display"""
        if hasattr(self, '_cached_url'):
            return getattr(self, '_cached_url')
        url = ''
        if self.url:
            url = self.url
        elif self.reverse:
            reverse_kwargs = {}
            if self.reverse_kwargs:
                args_keywords = self.reverse_kwargs.split(',')
                for arg_keyword in args_keywords:
                    if ':' in arg_keyword:
                        slices = arg_keyword.split(':')
                        keyword = slices[0]
                        default_value = ":".join(slices[1:])
                    else:
                        keyword = arg_keyword
                        default_value = None
                    reverse_kwargs[keyword] = kwargs.get(keyword, default_value)
            try:
                query_string = ''
                request = RequestManager().get_request()
                if request:
                    query_string = request.META['QUERY_STRING'] or ''
                try:
                    url = reverse(self.reverse, kwargs=reverse_kwargs) + "?" + query_string
                except TypeError:
                    pass
            except NoReverseMatch as err:
                if settings.DEBUG:
                    print("NoReverseMatch", err)

        if url:
            setattr(self, '_cached_url', url)
        return url


@python_2_unicode_compatible
class UserPermissions(models.Model):
    """user preferences"""

    user = models.OneToOneField(User)
    can_create_group = models.BooleanField(
        default=True, verbose_name=_('can create group'), help_text=_('If not checked, the user can not create group')
    )

    class Meta:
        verbose_name = _('User permissions')
        verbose_name_plural = _('User permissions')

    def __str__(self):
        return self.user.username
